#include <afx.h>
#include <winsock2.h>
#include <stdio.h>
#include <io.h>
#include <conio.h>
#include "../stdint.h"
#include <list>
#include "../dprintf.h"

using namespace std;

#define byte unsigned char
#define word unsigned short
#define dword unsigned int
#define fio_SECTOR_SIZE 2048

#define be2me_32(x) ((((x) & 0xff000000) >> 24) | (((x) & 0x00ff0000) >>  8) | (((x) & 0x0000ff00) <<  8) | (((x) & 0x000000ff) << 24))
#define get4bytes(b) be2me_32(*((dword *)(b)))

typedef struct
{
	uint8_t	hour;
	uint8_t	minute;
	uint8_t	second;
	uint8_t	frame_u;	// The two high bits are the frame rate
} dvd_time_t;

#define OFF_TITLE_PGCI get4bytes (ifo->mat + 0xCC)

typedef struct
{
	byte	*mat;
	byte	*title_pgci;
} ifo_t;

typedef struct
{
	uint16_t	num	: 16;	// number of entries
	uint16_t	reserved	: 16;	// don't known (reserved?)
	uint32_t	len	: 32;	// length of table
} ifo_hdr_t;

typedef struct
{
	uint16_t	id	: 16;	// Language
	uint16_t		: 16;	// don't know
	uint32_t	start	: 32;	// Start of unit
} pgci_sub_t;

/*
typedef struct
{ // Program Chain Information
	uint16_t	zero_1;
	uint8_t	nr_of_programs;
	uint8_t	nr_of_cells;
	dvd_time_t	playback_time;
	uint32_t	prohibited_ops;	// New type?
	audio_status_t	audio_status[IFO_NUM_AUDIO];
	subp_status_t	subp_status[IFO_NUM_SPU];
	uint16_t	next_pgc_nr;
	uint16_t	prev_pgc_nr;
	uint16_t	goup_pgc_nr;
	uint8_t	still_time;
	uint8_t	pg_playback_mode;
	clut_t	clut[16];
	uint16_t	pgc_command_tbl_offset;
	uint16_t	pgc_program_map_offset;
	uint16_t	cell_playback_tbl_offset;
	uint16_t	cell_position_tbl_offset;
} pgc_t;
*/
/*
const pgc_t *iifoGetPGCI(const ifo_t *ifo, const int title)
{
	pgci_sub_t *pgci_sub;
	ifo_hdr_t *hdr = NULL;
	uint8_t *ptr = NULL;
	
	hdr = (ifo_hdr_t *)ifo->title_pgci;
	
	if (hdr && title < be2me_16(hdr->num))
	{
		ptr = (uint8_t *)hdr;
		ptr += IFO_HDR_LEN;
		
		pgci_sub = (pgci_sub_t *)ptr + title;
		
		ptr = (uint8_t *)hdr + be2me_32(pgci_sub->start);
		int val = (uint8_t *)ptr - (uint8_t *)hdr;
		int s = sizeof(pgc_t)-8;
		int off = OFF_TITLE_PGCI; 
		int off2 = ptr - ifo->mat;
		unsigned char *ss = &ptr[s];
		const pgc_t *ppp =(const pgc_t *)ptr;
		int s3 = (unsigned char *)&ppp->clut[0] - (unsigned char *)hdr;
	}
	
	return (const pgc_t *)ptr;
}
*/

#define IFO_HDR_LEN 8

int GetClutAddress(const ifo_t *ifo, const int title)
{
	unsigned char *hdr = (unsigned char *)ifo->title_pgci;
	unsigned char *ptr = hdr;
	ptr += IFO_HDR_LEN;
	
	pgci_sub_t *pgci_sub = (pgci_sub_t *)ptr + title;

	ptr = (uint8_t *)hdr + be2me_32(pgci_sub->start);
	int ret = (ptr+164)-hdr;

	return ret+4096;
}

bool iifoIsVTS2(ifo_t *ifo)
{
	return strncmp ((const char *)ifo->mat, "DVDVIDEO-VTS", 12) == 0;
}

int iifoReadLB(FILE* fp, dword pos, dword count, byte *data)
{
	if ((pos = fseek(fp, pos, SEEK_SET)) == -1)
		return -1;
	return fread(data, count, 1, fp);
}

bool iifoReadTBL(FILE* fp, dword offset, byte **out_ptr)
{
	byte *data;
	ifo_hdr_t *hdr;
	dword len = 0;
	
	if (!offset)
		return FALSE;
	
	data =  (byte *)malloc(fio_SECTOR_SIZE);
	if (!data)
		return FALSE;
	
	if (iifoReadLB (fp, offset * fio_SECTOR_SIZE, fio_SECTOR_SIZE, data) <= 0)
		return FALSE;
	
	hdr = (ifo_hdr_t *)data;
	len = be2me_32(hdr->len) + 1;
	
	if (len > fio_SECTOR_SIZE)
	{
		if (!(data = (byte *)realloc((void *)data, len)))
			return FALSE;
		iifoReadLB (fp, offset * fio_SECTOR_SIZE, len, data);
	}
	*out_ptr = data;
	return TRUE;
}

ifo_t* ifoOpen(const char *name)
{
	ifo_t *ret_ifo = NULL;
	
	FILE *fd = fopen(name, "rb");
	if (fd)
	{
		ifo_t *ifo = (ifo_t *)calloc(sizeof *ifo, 1);
		
		ifo->mat = (unsigned char*)calloc(fio_SECTOR_SIZE, 1);
		if (iifoReadLB(fd, 0, fio_SECTOR_SIZE, ifo->mat) < 0)
		{
			free(ifo->mat);
			free(ifo);
		}
		else 
		{ // looking ok so far
			if (iifoIsVTS2(ifo) && iifoReadTBL(fd, OFF_TITLE_PGCI, &ifo->title_pgci))
				ret_ifo = ifo;
			else
			{
				if (ifo->title_pgci)
					free(ifo->title_pgci);
				free (ifo->mat);
				free (ifo);
			}
		}
		fclose(fd);
	}
	return ret_ifo;
}

bool ifoClose2 (ifo_t *ifo)
{
	if (ifo)
	{
		if (ifo->mat)
			free (ifo->mat);
		if (ifo->title_pgci)
			free (ifo->title_pgci);
		free (ifo);
	}
	return TRUE;
}

void ClutCopy(LPCSTR szInIFO, int nInPGC, LPCSTR szOutIFO, int nOutPGC)
{
	printf("CLUTCopy by Nic\n");

	ifo_t *ifo = ifoOpen(szInIFO);
	if ( !ifo )
	{
		dprintf("Couldn't Open: %s \n", szInIFO);
		return;
	}
	int offset1 = GetClutAddress(ifo, nInPGC);
	ifoClose2(ifo);

	byte clut[16*4];
	
	FILE *fin = fopen(szInIFO, "rb");
	if ( !fin )
	{
		dprintf("Couldn't Open: %s \n", szInIFO);
		return;
	}
	fseek(fin, offset1, SEEK_SET);
	fread(clut, 16*4, 1, fin);
	fclose(fin);

	ifo = ifoOpen(szOutIFO);
	if ( !ifo )
	{
		printf("Couldn't Open: %s \n", szOutIFO);
		return;
	}

	int offset2 = GetClutAddress(ifo, nOutPGC);
	ifoClose2(ifo);

	FILE *fout = fopen(szOutIFO, "r+b");
	if ( !fout )
	{
		printf("Couldn't Open: %s \n", szOutIFO);
		return;
	}
	fseek(fout, offset2, SEEK_SET);
	fwrite(clut, 16*4, 1, fout);
	fclose(fout);

	// Now also copy to BUP if it exists
	CString csBUPFile = szOutIFO;

	csBUPFile.SetAt(csBUPFile.GetLength()-3, 'B');
	csBUPFile.SetAt(csBUPFile.GetLength()-2, 'U');
	csBUPFile.SetAt(csBUPFile.GetLength()-1, 'P');

	fout = fopen(csBUPFile, "r+b");
	if ( !fout )
	{
		printf("Couldn't Open: %s \n", csBUPFile);
		return;
	}
	fseek(fout, offset2, SEEK_SET);
	fwrite(clut, 16*4, 1, fout);
	fclose(fout);

	printf("Done.\n");
}