
#include "../stdafx.h"
#include <winsock2.h>
#include <stdio.h>
#include <io.h>
#include <conio.h>
#include "../stdint.h"
#include <fcntl.h>
#include <list>
#include <set>
#include "../Authoring.h"

using namespace std;

unsigned int BitsLeft;
unsigned int CurrentBfr;

#define PS_HEADER_L1    14
#define PS_HEADER_L2    (PS_HEADER_L1+24)
#define PES_MIN         7
#define PES_H_MIN       9

#define SYSTEM_START_CODE_S   0xB9
#define SYSTEM_START_CODE_E   0xFF
#define PACK_START            0xBA
#define SYS_START             0xBB
#define PROG_STREAM_MAP       0xBC
#define PRIVATE_STREAM1       0xBD
#define PADDING_STREAM        0xBE
#define PRIVATE_STREAM2       0xBF
#define AUDIO_STREAM_S        0xC0
#define AUDIO_STREAM_E        0xDF
#define VIDEO_STREAM_S        0xE0
#define VIDEO_STREAM_E        0xEF
#define ECM_STREAM            0xF0
#define EMM_STREAM            0xF1
#define DSM_CC_STREAM         0xF2
#define ISO13522_STREAM       0xF3
#define PROG_STREAM_DIR       0xFF

//flags2
#define PTS_DTS_FLAGS    0xC0
#define ESCR_FLAG        0x20
#define ES_RATE_FLAG     0x10
#define DSM_TRICK_FLAG   0x08
#define ADD_CPY_FLAG     0x04
#define PES_CRC_FLAG     0x02
#define PES_EXT_FLAG     0x01

//pts_dts flags
#define PTS_ONLY         0x80
#define PTS_DTS          0xC0

#define MAX_PLENGTH 0xFFFF
#define MMAX_PLENGTH (8*MAX_PLENGTH)

#define MAX_PTS (0x0000000200000000)
#define MAX_PTS2 (300* MAX_PTS)

typedef
struct ps_packet_{
	uint8_t scr[6];
	uint8_t mux_rate[3];
	uint8_t stuff_length;
	uint8_t *data;
	uint8_t sheader_llength[2];
	int sheader_length;
	uint8_t rate_bound[3];
	uint8_t audio_bound;
	uint8_t video_bound;
	uint8_t reserved;
	int npes;
} ps_packet;

void init_ps(ps_packet *p)
{
        p->stuff_length=0xF8;
        p->data = NULL;
        p->sheader_length = 0;
        p->audio_bound = 0;
        p->video_bound = 0;
        p->npes = 0;
}

void kill_ps(ps_packet *p)
{
        if (p->data)
                free(p->data);
        init_ps(p);
}

void setlength_ps(ps_packet *p)
{
        short *ll;
        ll = (short *) p->sheader_llength;
	p->sheader_length = ntohs(*ll) - 6;
}

static void setl_ps(ps_packet *p)
{
        setlength_ps(p);
        p->data = (uint8_t *) malloc(p->sheader_length);
}

void printpts(__int64 pts)
{
	if (pts < 0){
		fprintf(stderr,"-");
		pts = -pts;
	}
	///pts = pts;
	//pts &= (MAX_PTS-1);
	fprintf(stderr,"%2d:%02d:%02d.%03d ",
		(unsigned int)(pts/90000.)/3600,
		((unsigned int)(pts/90000.)%3600)/60,
		((unsigned int)(pts/90000.)%3600)%60,
		(((unsigned int)(pts/90.)%3600000)%60000)%1000
		);
}

unsigned int Get_Bits2(unsigned int N)
{
		int Val = (CurrentBfr << (32 - BitsLeft)) >> (32 - N);
		BitsLeft -= N;
		return Val;
}

void get_pespts(uint8_t *spts,uint8_t *pts)
{

        pts[0] = 0x21 |
                ((spts[0] & 0xC0) >>5);
        pts[1] = ((spts[0] & 0x3F) << 2) |
                ((spts[1] & 0xC0) >> 6);
        pts[2] = 0x01 | ((spts[1] & 0x3F) << 2) |
                ((spts[2] & 0x80) >> 6);
        pts[3] = ((spts[2] & 0x7F) << 1) |
                ((spts[3] & 0x80) >> 7);
        pts[4] = 0x01 | ((spts[3] & 0x7F) << 1);
}

int write_pes_header(uint8_t id, int length , uint64_t PTS, uint64_t DTS, 
		     uint8_t *obuf, int stuffing, uint8_t ptsdts, bool bExt)
{
	uint8_t le[2];
	uint8_t dummy[3];
	uint8_t *pts;
	uint8_t ppts[5];
	uint32_t lpts;
	uint8_t *dts;
	uint8_t pdts[5];
	uint32_t ldts;
	int c;
	uint8_t headr[3] = {0x00, 0x00, 0x01};
	
	lpts = htonl((PTS/300) & 0x00000000FFFFFFFF);
	pts = (uint8_t *) &lpts;
	get_pespts(pts,ppts);
	if ((PTS/300) & 0x0000000100000000) ppts[0] |= 0x80;

	ldts = htonl((DTS/300) & 0x00000000FFFFFFFF);
	dts = (uint8_t *) &ldts;
	get_pespts(dts,pdts);
	if ((DTS/300) & 0x0000000100000000) pdts[0] |= 0x80;

	c = 0;
	memcpy(obuf+c,headr,3);
	c += 3;
	memcpy(obuf+c,&id,1);
	c++;

	le[0] = 0;
	le[1] = 0;
	length -= 6;

	le[0] |= ((uint8_t)(length >> 8) & 0xFF); 
	le[1] |= ((uint8_t)(length) & 0xFF); 
	memcpy(obuf+c,le,2);
	c += 2;

	if (id == PADDING_STREAM){
		memset(obuf+c,0xff,length);
		c+= length;
		return c;
	}

	dummy[0] = 0x80;
	dummy[1] = 0;
	dummy[2] = 0;
	
	if (ptsdts == PTS_ONLY){
		dummy[2] = 5 + stuffing;
		dummy[1] |= PTS_ONLY;
	} else 	if (ptsdts == PTS_DTS){
		dummy[2] = 10 + stuffing;
		dummy[1] |= PTS_DTS;
	}

	// Add Extension
	if ( bExt )
	{
		dummy[1] |= 1;
		dummy[2] += 3;
	}

	memcpy(obuf+c,dummy,3);
	c += 3;

	memset(obuf+c,0xFF,stuffing);
	c += stuffing;

	if (ptsdts == PTS_ONLY){
		memcpy(obuf+c,ppts,5);
		c += 5;
	} else if ( ptsdts == PTS_DTS ){
		memcpy(obuf+c,ppts,5);
		c += 5;
		memcpy(obuf+c,pdts,5);
		c += 5;
	}

	// Add Extension
	if ( bExt )
	{
		unsigned char exthdr[16];
		// first 3 bits should 001
		exthdr[0] = 0x10;
		exthdr[1] = 0x60;	// Just set P-STD flag
		exthdr[2] = 0x3a;	//58;
		memcpy(obuf+c,exthdr,3);
		c += 3;
	}

	return c;
}

int cwrite_ps(uint8_t *buf, ps_packet *p, uint32_t length)
{
        long count,i;
        uint8_t headr1[4] = {0x00, 0x00, 0x01, PACK_START };
        uint8_t headr2[4] = {0x00, 0x00, 0x01, SYS_START };
        uint8_t buffy = 0xFF;


        memcpy(buf,headr1,4);
        count = 4;
	memcpy(buf+count,p->scr,6);
	count += 6;
	memcpy(buf+count,p->mux_rate,3);
	count += 3;
	memcpy(buf+count,&p->stuff_length,1);
	count++;
	for(i=0; i< (p->stuff_length & 3); i++){
		memcpy(buf+count,&buffy,1);
		count++;
	}

        if (p->sheader_length){
                memcpy(buf+count,headr2,4);
                count += 4;
                memcpy(buf+count,p->sheader_llength,2);
                count += 2;
		memcpy(buf+count,p->rate_bound,3);
		count += 3;
		memcpy(buf+count,&p->audio_bound,1);
		count++;
		memcpy(buf+count,&p->video_bound,1);
		count++;
		memcpy(buf+count,&p->reserved,1);
		count++;
                memcpy(buf+count,p->data,p->sheader_length);
                count += p->sheader_length;
        }

        return count;
}



int write_ps_header(uint8_t *buf, 
		    uint64_t   SCR, 
		    uint32_t   muxr,
		    uint8_t    audio_bound,
		    uint8_t    fixed,
		    uint8_t    CSPS,
		    uint8_t    audio_lock,
		    uint8_t    video_lock,
		    uint8_t    video_bound,
		    uint8_t    navpack)
{
	ps_packet p;
	uint8_t *scr;
	uint32_t lscr;
	uint16_t scr_ext = 0;

	init_ps(&p);
	
	lscr = htonl((uint32_t) ((SCR/300) & 0x00000000FFFFFFFF));
	scr = (uint8_t *) &lscr;
	scr_ext = (uint16_t) ((SCR%300) & 0x00000000000001FF);
	
// SCR = 0
	p.scr[0] = 0x44;
	p.scr[1] = 0x00;
	p.scr[2] = 0x04;
	p.scr[3] = 0x00;
	p.scr[4] = 0x04;
	p.scr[5] = 0x01;
	
	p.scr[0] = 0x44 | ((scr[0] >> 3)&0x18) | ((scr[0] >> 4)&0x03);
	p.scr[1] = 0x00 | ((scr[0] << 4)&0xF0) | ((scr[1] >> 4)&0x0F);
	p.scr[2] = 0x04 | ((scr[1] << 4)&0xF0) | ((scr[2] >> 4)&0x08)
		| ((scr[2] >> 5)&0x03);
	p.scr[3] = 0x00 | ((scr[2] << 3)&0xF8) | ((scr[3] >> 5)&0x07);
	p.scr[4] = 0x04 | ((scr[3] << 3)&0xF8) | ((scr_ext >> 7)&0x03);
	p.scr[5] = 0x01 | ((scr_ext << 1)&0xFF);

	
	muxr = muxr/50;
	p.mux_rate[0] = (uint8_t)(muxr >> 14);
	p.mux_rate[1] = (uint8_t)(0xff & (muxr >> 6));
	p.mux_rate[2] = (uint8_t)(0x03 | ((muxr & 0x3f) << 2));

	p.stuff_length = 0xF8;
	
	if (navpack){
		p.sheader_llength[0] = 0x00;
		p.sheader_llength[1] = 0x12;

		setl_ps(&p);
		
		p.rate_bound[0] = (uint8_t)(0x80 | (muxr >>15));
		p.rate_bound[1] = (uint8_t)(0xff & (muxr >> 7));
		p.rate_bound[2] = (uint8_t)(0x01 | ((muxr & 0x7f)<<1));

	
		p.audio_bound = (uint8_t)((audio_bound << 2)|(fixed << 1)|CSPS);
		p.video_bound = (uint8_t)((audio_lock << 7)|
				     (video_lock << 6)|0x20|video_bound);
		p.reserved = (uint8_t)(0xFF >> 1);

		p.data[0] = 0xB9;  
		p.data[1] = 0xE0;  
		p.data[2] = 0xE8;  
		p.data[3] = 0xB8;  
		p.data[4] = 0xC0;  
		p.data[5] = 0x20;  
		p.data[6] = 0xbd;  
		p.data[7] = 0xe0;  
		p.data[8] = 0x3a;  
		p.data[9] = 0xBF;  
		p.data[10] = 0xE0;  
		p.data[11] = 0x02;  

		cwrite_ps(buf, &p, PS_HEADER_L2);
		kill_ps(&p);
		return PS_HEADER_L2;
	} else {
		cwrite_ps(buf, &p, PS_HEADER_L1);
		kill_ps(&p);
		return PS_HEADER_L1;
	}
}

int write_sub_pes( int n, uint64_t pts, uint64_t SCR, 
		    uint32_t muxr, uint8_t *buf, int *alength, uint8_t ptsdts, unsigned char *pBuf, bool bFirst)
{
	int add;
	int pos = 0;
	int p   = 0;
	int stuff = 0;
	int length = *alength+1;	// to account for the 0x80
	int pack_size = 2048;

	if (!length) return 0;
	p = PS_HEADER_L1+PES_H_MIN;

	if (ptsdts == PTS_ONLY){
		p += 5;
	}

	if ( bFirst )
		p += 3;

	if ( length+p >= pack_size){
		length = pack_size;
	} else {
		if (pack_size-length-p <= PES_MIN){
			stuff = pack_size - length-p;
			length = pack_size;
		} else 
			length = length+p;
	}
	pos = write_ps_header(buf,SCR,muxr, /*rx->apidn+rx->ac3n+0x20-n+1*/1, 0, 0, 1, 1, 
			      1, 0);

	pos += write_pes_header( PRIVATE_STREAM1, length-pos, pts, 0, 
				 buf+pos, stuff, ptsdts, bFirst);

	buf[pos] = n;
	pos += 1;

	if ( length-pos <= *alength )
	{
		memcpy(buf+pos, pBuf, length-pos);
		add = length-pos;
	}
	else
	{
		memcpy(buf+pos, pBuf, *alength);
		add = *alength;
	}

	*alength = add;
	if (add < 0) return -1;
	pos += add;

	if (pos+PES_MIN < pack_size){
		pos += write_pes_header( PADDING_STREAM, pack_size-pos, 0,0,
					 buf+pos, 0, 0, false);
		pos = pack_size;
	}		
	if (pos != pack_size) {
		fprintf(stderr,"apos: %d\n",pos);
		exit(1);
	}

	return pos;
}

class CSUPBuf
{
public:
	int					nSubStreamNo;
	int					nSize;
	unsigned char		*pBuf;
	unsigned __int64	PTS;

	bool operator<(const CSUPBuf& Buf) const
	{
		return PTS < Buf.PTS;
	}
};

#define SWAPSHORT(x) ((( x >> 8 ) | ( (x & 0x00ff) << 8 )))

void SupMux(int argc, char* argv[], HWND hAuthorProgressBar, CDialog *pAuthorCWnd, HANDLE hAuthorStop)
{
	list<CSUPBuf>		   lstSubBuffers;
	set<int>			   setStreamsUsed;

	if ( argc < 4 )
	{
		printf("USAGE: SupMux Input.vob Output.vob input1.sup 0x20 input2.sup 0x21 ... etc\n");
		return;
	}

	m_hAuthorProgressBar = hAuthorProgressBar;
	m_pAuthorCWnd = pAuthorCWnd;
	m_hAuthorStop = hAuthorStop;

	// Get Input Size
	unsigned __int64 nTotalSize = 0;
	int finsize = _open(argv[1], _O_RDONLY | _O_BINARY);
	if ( finsize > 0 )
	{
		_lseeki64(finsize, 0, SEEK_END);
		nTotalSize = (unsigned __int64)_telli64(finsize);
		_close(finsize);
	}

	FILE *fin = fopen(argv[1], "rb");
	FILE *fout = fopen(argv[2], "wb");

	if ( !fin )
	{
		printf("Couldn't Open %s \n",argv[1]);
		return;
	}
	if ( !fout )
	{
		fclose(fin);
		printf("Couldn't Open %s \n",argv[1]);
		return;
	}

	for ( int i = 3; i < argc; i+=2 )
	{
		// Load in SUPs
		FILE *fsup = fopen(argv[i], "rb");

		if ( !fsup )
		{
			fclose(fin);
			fclose(fout);
			printf("Couldn't Open %s \n",argv[i]);
			return;
		}

		unsigned __int64 nPTS;
		unsigned short nSize;
		char szSP[2];

		int nSubStreamNo = 0x20;
		sscanf(argv[i+1], "%X", &nSubStreamNo);

		while ( true )
		{
			int nRead = fread(szSP, 1, 2, fsup);
			if ( nRead != 2 )
				break;
			if ( !(szSP[0] == 'S' &&
				 szSP[1] == 'P') )
			{
				printf("SUP Error!");
				break;
			}
			fread(&nPTS, sizeof(unsigned __int64), 1, fsup);
			fread(&nSize, sizeof(unsigned short), 1, fsup);	
			nSize = SWAPSHORT(nSize);
			fseek(fsup, -2, SEEK_CUR);
			unsigned char *subbuf = new unsigned char[nSize];
			fread(subbuf, nSize, 1, fsup);
			CSUPBuf buf;
			buf.nSize = nSize;
			buf.pBuf = subbuf;
			buf.nSubStreamNo = nSubStreamNo;
			buf.PTS = nPTS;
			lstSubBuffers.push_back(buf);
		}
		
		fclose(fsup);
	}

	lstSubBuffers.sort();

	unsigned char buf[2048];
	__int64 nFirstVideoPTS = -1;
	unsigned __int64 nAmount = 0;
	int count = 0;
	
	printf("SupMux - By Nic\nMuxing Sups into VOB:\n");
	
	while ( fread(buf, 2048, 1, fin) > 0 )
	{
		nAmount += 2048;
		if ( !(count++ % 200) )
		{
			//printf("%02d%%  \r", (int)((((double)nAmount)/((double)nTotalSize))*100.0));
			//fflush(stdout);
			if ( WaitForSingleObject(m_hAuthorStop, 0) != WAIT_TIMEOUT )
			{
				break;
			}

			int nPercent = (int)((((double)nAmount)/((double)nTotalSize))*100.0);
			SendMessage(m_hAuthorProgressBar, PBM_SETPOS, nPercent, 0);
			
			MSG msg;
			while (PeekMessage(&msg, NULL, 0, 0, PM_REMOVE))
			{
				if ( msg.message == WM_QUIT  )
				{
					 break;
				}

				TranslateMessage(&msg);
				DispatchMessage(&msg);
			}
		}

		if ( buf[3] == 0xba && buf[0x11] == 0xe0 )
		{
			// Video Stream
			// Skip 3
			int nPTS = (buf[21] & 0x80);
			if ( nPTS )
			{	
				// read SCR
				__int64 SCR;
				__int64 SCR_32, SCR_29, SCR_14;

				CurrentBfr = ( buf[4] << 24 ) |
							 ( buf[5] << 16 ) |
							 ( buf[6] << 8  ) |
							 ( buf[7] );
				BitsLeft   = 32;

				BitsLeft -= 2;		// skip 01
				SCR_32 = Get_Bits2(3);
				BitsLeft--;
				SCR_29 = Get_Bits2(15);
				BitsLeft--;


				CurrentBfr = ( buf[6] << 24 ) |
							 ( buf[7] << 16 ) |
							 ( buf[8] << 8  ) |
							 ( buf[9] );
				BitsLeft   = 32;
				BitsLeft   -= 6;

				SCR_14 = Get_Bits2(15);

				SCR = SCR_32 << 30; 
				SCR |= ( SCR_29 << 15 ); 
				SCR |= SCR_14; 

				// Get Muxrate
				CurrentBfr = ( buf[10] << 24 ) |
							 ( buf[11] << 16 ) |
							 ( buf[12] << 8  ) |
							 ( buf[13] );
				BitsLeft   = 32;

				unsigned int muxr;
				muxr = Get_Bits2(22);

				// Replex tends to need muxr*50 and pts*300

				// Read PTS values
				int i = 14;
				CurrentBfr = ( buf[i+9] << 24 ) |
							 ( buf[i+10] << 16 ) |
							 ( buf[i+11] << 8  ) |
							 ( buf[i+12] );
				BitsLeft   = 32;
				int code = Get_Bits2(4);
				//BitsLeft   -= 4;
				
				__int64 Video_PTS;
				__int64 PTS_32, PTS_29, PTS_14;
				PTS_32 = Get_Bits2(3);
				BitsLeft--;
				PTS_29 = Get_Bits2(15);
				BitsLeft--;
				CurrentBfr = ( buf[i+12] << 24 ) |
							 ( buf[i+13] << 16 ) |
							 ( buf[i+14] << 8  ) |
							 ( buf[i+15] );
				BitsLeft   = 32;
				PTS_14 = Get_Bits2(15);
				Video_PTS = PTS_32 << 30; 
				Video_PTS |= ( PTS_29 << 15 ); 
				Video_PTS |= PTS_14; 

				if ( nFirstVideoPTS == -1 )
					nFirstVideoPTS = Video_PTS;

				// Check SUPs 
				//printf("VideoPTS: %I64u SubPTS: %I64u PlusFirst: %I64u...", Video_PTS, lstSubBuffers.front().PTS, lstSubBuffers.front().PTS+nFirstVideoPTS);
				while ( Video_PTS > lstSubBuffers.front().PTS+nFirstVideoPTS )
				{
				//	printf("Used..");
					unsigned char outbuf[4096];

					CSUPBuf buf = lstSubBuffers.front();

					int nSize = buf.nSize;
					unsigned char *pBuf = buf.pBuf;

					bool bWriteExt = false;
					if ( setStreamsUsed.find(buf.nSubStreamNo) == setStreamsUsed.end() )
						bWriteExt = true;

					bool bFirst = true;
					while ( nSize )
					{
						int nlength = nSize;
						int written = write_sub_pes(buf.nSubStreamNo, (nFirstVideoPTS+buf.PTS)*300, SCR*300, muxr*50, outbuf, &nlength, bFirst == true ? PTS_ONLY : 0, pBuf, bWriteExt);
						fwrite(outbuf, written, 1, fout);
						pBuf += nlength;
						nSize -= nlength;
						bFirst = false;
						bWriteExt = false;
					}

					delete [] buf.pBuf;
					setStreamsUsed.insert(buf.nSubStreamNo);
					lstSubBuffers.pop_front();
				}	
			}

		}
		
		// Write out the PES
		fwrite(buf, 2048, 1, fout);
	}

	printf("\nDone.\n");
	fclose(fin);
	fclose(fout);
}