// delayac3.cpp : Defines the entry point for the console application.
//
// version 0.27
//
#include "stdafx.h"
#include "sil48.h"
#include "stdio.h"
#include "string.h"
#include "io.h"
#include "sys\types.h"
#include "sys\stat.h"

#define version "0.27"
#define NUMREAD 8
#define MAXBYTES_PER_FRAME 2*1280
#define debug  0
#define CRC16_POLY ((1 << 0) | (1 << 2) | (1 << 15) | (1 << 16))

// actions in crc errors
#define CRC_IGNORE 0
#define CRC_FIX 1
#define CRC_SILENCE 2
#define CRC_SKIP 3

//actions to write frame
#define WF_SKIP 0
#define WF_WRITE 1
#define WF_SILENCE 2

unsigned long int p_bit;
unsigned int crc_table[256];


////////////////////////////////////////////////////////////////////////
//      ac3_crc_init
////////////////////////////////////////////////////////////////////////

static void ac3_crc_init(void)
{
    unsigned int c, n, k;

    for(n=0;n<256;n++) {
        c = n << 8;
        for (k = 0; k < 8; k++) {
            if (c & (1 << 15)) 
                c = ((c << 1) & 0xffff) ^ (CRC16_POLY & 0xffff);
            else
                c = c << 1;
        }
        crc_table[n] = c;
    }
}

////////////////////////////////////////////////////////////////////////
//      ac3_crc: Calculates the crc
////////////////////////////////////////////////////////////////////////
static unsigned int ac3_crc(unsigned int *data, int n, unsigned int crc)
{
    int i;
    for(i=0;i<n;i++) {
        crc = (crc_table[(data[i] & 0xff)^ (crc >> 8)] ^ (crc << 8)) & 0xffff;
    }
    return crc;
}

////////////////////////////////////////////////////////////////////////
//      mul_poply and pow_poly: Operations with polynomials
////////////////////////////////////////////////////////////////////////
static unsigned int mul_poly(unsigned int a, unsigned int b, unsigned int poly)
{
    unsigned int c;

    c = 0;
    while (a) {
        if (a & 1)
            c ^= b;
        a = a >> 1;
        b = b << 1;
        if (b & (1 << 16))
            b ^= poly;
    }
    return c;
}

static unsigned int pow_poly(unsigned int a, unsigned int n, unsigned int poly)
{
    unsigned int r;
    r = 1;
    while (n) {
        if (n & 1)
            r = mul_poly(r, a, poly);
        a = mul_poly(a, a, poly);
        n >>= 1;
    }
    return r;
}

////////////////////////////////////////////////////////////////////////
//      writeframe
////////////////////////////////////////////////////////////////////////

unsigned int nBytesWritten = 0;

void writeframe (FILE *fileout, unsigned int *p_frame)
{
	unsigned long int i;
	unsigned long int frmsizecod, nubytes;

	frmsizecod=p_frame[4] & 0x3F;
	nubytes=FrameSize_48[frmsizecod]*2;
	nBytesWritten+=nubytes;

	for (i=0;i < nubytes ;i++)
	{
		fputc(p_frame[i],fileout);
	}

	return;
}

////////////////////////////////////////////////////////////////////////
//      readframe
////////////////////////////////////////////////////////////////////////

int readframe (FILE *filein, unsigned int *p_frame)
{
	int i, frmsizecod,BytesPerFrame;

	p_bit=0;
	for (i=0;!feof(filein) && i < NUMREAD ;i++)
	{
		p_frame[i]=fgetc(filein);
	}


	frmsizecod=p_frame[4] & 0x3F;
	BytesPerFrame=FrameSize_48[frmsizecod]*2;
	for (i=NUMREAD;!feof(filein) && i < BytesPerFrame ;i++)
	{
		p_frame[i]=fgetc(filein);
	}

//	if (i != BytesPerFrame)
//	{
//          Reached End Of File: Truncated 
//		return -1;
//	}
	
	return i;
}

////////////////////////////////////////////////////////////////////////
//      getbits
////////////////////////////////////////////////////////////////////////
unsigned int getbits (int number, unsigned int *p_frame)
{
	unsigned int bit_ini,byte_ini,bit_end,byte_end, output;

	byte_ini=p_bit/8;
	bit_ini=p_bit%8;

	p_bit+=number;

	byte_end=p_bit/8;
	bit_end=p_bit%8;

	if (byte_end==byte_ini)
		output= p_frame[byte_end];
	else if (byte_end==byte_ini+1)
		output= p_frame[byte_end-1]*256 + p_frame[byte_end];
	else
	    output= p_frame[byte_end-2]*256*256+p_frame[byte_end-1]*256+
		      p_frame[byte_end];
		  
	output=(output)>>(8-bit_end);
	output=output & ((1 << number)-1);
    
	return output;
}

////////////////////////////////////////////////////////////////////////
//      usage
////////////////////////////////////////////////////////////////////////
void usage()
{
	// fprintf (stderr,"Usage: delayac3 [options] <filein>\n");
	// fprintf (stderr,"  Options\n");
	// fprintf (stderr,"  -info: Do nothing, only outputs info in log file\n");
	// fprintf (stderr,"  -fixcrc {ignore, skip, fix, silence}: Action in the case\n");
	// fprintf (stderr,"                        of crc errors\n");
	// fprintf (stderr,"  -start <delay_msecs>: Adds (-Cuts) the needed frames \n");
	// fprintf (stderr,"                        at the beginning\n");
	// fprintf (stderr,"  -same: Keeps original duration \n");
	// fprintf (stderr,"  -startsplit <ini_sec>: Starts in ini_sec \n");
	// fprintf (stderr,"  -endsplit <end_sec>:  Stops in end_sec \n");

	return;
}

////////////////////////////////////////////////////////////////////////
//
//      Main
//
////////////////////////////////////////////////////////////////////////

CString DelayAC3(const char *szAC3File, double fDelay, CProgressCtrl *pProgressBar, HANDLE hStop, CDialog *pDlg)
{

	FILE *in, *fix;

// BSI variables
	unsigned long int syncword, crc1, fscod,frmsizecod;
	unsigned long int bsid, bsmod, acmod, surmixlev, cmixlev, dsurmod;
	unsigned long int lfeon,dialnorm,compre, compr;

	unsigned long int syncwordn, crc1n, fscodn,frmsizecodn;
	unsigned long int bsidn, bsmodn, acmodn;

// Delay control
	__int64 StartFramesAdded, EndFramesAdded;
	double delay, delay2, NotFixedDelay;

// index & switches
	__int64 i;
	int	info, same, fixcrc, f_writeframe;

// frame arrays
	unsigned int caracter[2600], silence[2600];

// other variables
	__int64  nuframes, TimeLengthIni, TimeLengthEst,TimeLengthEstSplit, TimeLengthFix, rest, nuframeswritten;
	unsigned long int nubytes,rate, BytesPerFrame, BytesPerFramen;
	char              mode[10];
	double            fsample, FrameDuration, FramesPerSecond;
	double		      startsplit,endsplit;
	struct            _stati64 statbuf;
	char              s_TimeLengthIni[120], s_TimeLengthEst[120], s_TimeLengthEstSplit[120], s_TimeLengthFix[120];
	char              Filename[MAX_PATH], FixedFilename[MAX_PATH];
	unsigned int *p_silence;
	unsigned int frame_size, frame_size_58, cal_crc2, cal_crc1, crc_inv;
    int maxerrorstatus, nuerrors;
    char s_time[100], s_fixcrc[20];

	nBytesWritten = 0;

// Start of the code
// init ac3_crc
	ac3_crc_init();

// Check arguments
	info=same=0;
	delay=0;
    maxerrorstatus=nuerrors=0;

	// Normally ignores
	fixcrc=CRC_IGNORE;
	strcpy(s_fixcrc,"IGNORED");
    
	startsplit=endsplit=0;

	// NICK
	// Set the delay
	delay = fDelay;
	fixcrc=CRC_FIX; strcpy(s_fixcrc,"FIXED");
	strcpy(Filename, szAC3File);
	strcpy (FixedFilename, Filename);
	FixedFilename[strlen(FixedFilename)-4]=0;
	strcat (FixedFilename,"_fixed.ac3");

/*
	for (n=1;n<argc-1;n++)
	{
		if      (strcmp(argv[n],"-info")==0) info=1;
		else if (strcmp(argv[n],"-same")==0) same=1;
		else if (strcmp(argv[n],"-start")==0)
		{
			n++;
			if (sscanf(argv[n], "%lf", &delay)!=1)
			{
				usage();
				return -1;
			}
		}
		else if (strcmp(argv[n],"-startsplit")==0)
		{
			n++;
			if (sscanf(argv[n], "%lf", &startsplit)!=1)
			{
				usage();
				return -1;
			}
		}
		else if (strcmp(argv[n],"-endsplit")==0)
		{
			n++;
			if (sscanf(argv[n], "%lf", &endsplit)!=1)
			{
				usage();
				return -1;
			}
		}
		else if (strcmp(argv[n],"-fixcrc")==0) 
		{
			n++;
			if      (strcmp(argv[n],"ignore")==0)  {fixcrc=CRC_IGNORE; strcpy(s_fixcrc,"IGNORED");}
			else if (strcmp(argv[n],"fix")==0)     {fixcrc=CRC_FIX; strcpy(s_fixcrc,"FIXED");}
			else if (strcmp(argv[n],"skip")==0)    {fixcrc=CRC_SKIP; strcpy(s_fixcrc,"SKIPPED");}
			else if (strcmp(argv[n],"silence")==0) {fixcrc=CRC_SILENCE; strcpy(s_fixcrc,"SILENCED");}
			else
			{
				usage();
				return -1;
			}
		}
	}
*/
/*	if (startsplit !=0 || endsplit != 0)
		if (startsplit<0 || endsplit <0 || ( endsplit != 0 && endsplit<startsplit))
		{
			usage();
			return -1;
		}


// Input and Output filenames
	strcpy(Filename,argv[argc-1]);
	if ( (strlen(Filename) <4 )  ||
		 (Filename[strlen(Filename)-4] != '.'))
		strcat(Filename,".ac3");

	strcpy (FixedFilename, Filename);
	FixedFilename[strlen(FixedFilename)-4]=0;
	strcat (FixedFilename,"_fixed.ac3");
*/
	if ((in=fopen(Filename,"rb"))==NULL)
	{
		// fprintf (stderr,"Unable to open input file: %s\n",Filename);
		return -1;
	}
	
	// No Log File
/*	if ((out=fopen("delayac3.log","w"))==NULL)
	{
		fclose (in);
		// fprintf (stderr,"Unable to open log file: delayac3.log\n");
		return -1;
	}
*/
// Read first frame
	nubytes=readframe (in, caracter);
	frmsizecod=caracter[4]& 0x3F;
	BytesPerFrame=FrameSize_48[frmsizecod]*2;
	while ( caracter[0]!= 0x0B || caracter[1]!=0x77 || 
		    nubytes < 12 || nubytes != BytesPerFrame)
	{
// Try to find the next sync Word and continue...
			// rewind nubytes (last frame)
		fseek(in, (long)(-1*nubytes), SEEK_CUR);
		caracter[0]=fgetc(in);
		caracter[1]=fgetc(in);
		while ((caracter[0]!= 0x0B || caracter[1]!=0x77) && !feof(in) )
		{
 			caracter[0]= caracter[1];
 			caracter[1]= fgetc(in);
		}
		if (caracter[0]== 0x0B && caracter[1]==0x77) 
		{
			// rewind 2 bytes
			fseek(in, -2L, SEEK_CUR);
			nubytes=readframe (in, caracter);
			frmsizecod=caracter[4]& 0x3F;
			BytesPerFrame=FrameSize_48[frmsizecod]*2;
		}
//		else:  nothimg to do, reached end of file
	}
//""""""""""
	if (nubytes <10 || caracter[0]!= 0x0B || caracter[1]!=0x77)
	{
		// fprintf (stdout,"Error: %s is not an ac3 stream\n",Filename);
		// fprintf (out,"Error: %s is not an ac3 stream\n",Filename);
//		fclose (out);
		return -1;
	}
	surmixlev=cmixlev=dsurmod=compr=0;
	syncword=   getbits (16, caracter);
	crc1=       getbits (16, caracter);
	fscod=      getbits (2,  caracter);
	frmsizecod= getbits (6,  caracter);
	bsid=       getbits (5,  caracter);
	bsmod=      getbits (3,  caracter);
	acmod=      getbits (3,  caracter);
	if ((acmod & 0x01) && (acmod != 0x01)) cmixlev=getbits(2, caracter);
	if (acmod & 0x4)	surmixlev=getbits(2, caracter);
	if (acmod == 0x2) dsurmod=getbits(2, caracter);
	lfeon=      getbits(1, caracter);
	dialnorm=   getbits(5, caracter);
	compre=     getbits(1, caracter);
	if (compre)	compr=getbits(8, caracter);


	if (fscod==0)		fsample=48.0;
	else if (fscod==1)	fsample=44.1;
	else if (fscod==2)	fsample=32.0 ;
	else fsample=0.0;

	if      (acmod==0) strcpy (mode,"1+1");
	else if (acmod==1) strcpy (mode,"1/0");
	else if (acmod==2) strcpy (mode,"2/0");
	else if (acmod==3) strcpy (mode,"3/0");
	else if (acmod==4) strcpy (mode,"2/1");
	else if (acmod==5) strcpy (mode,"3/1");
	else if (acmod==6) strcpy (mode,"2/2");
	else if (acmod==7) strcpy (mode,"3/2");

	rate=bitrate[frmsizecod];
 
	_fstati64 (fileno(in), &statbuf);

	BytesPerFrame=  FrameSize_48[frmsizecod]*2;
	nuframes=       statbuf.st_size/BytesPerFrame;
	rest=           statbuf.st_size%nuframes;
	FramesPerSecond=((double)(rate))*1000.0/(BytesPerFrame * 8);
	FrameDuration=  1000.0/FramesPerSecond; // in msecs
	TimeLengthIni=	statbuf.st_size/(rate/8); // in msecs
	sprintf (s_TimeLengthIni, "%02I64d:%02I64d:%02I64d.%03I64d",
		TimeLengthIni/(3600000), (TimeLengthIni%3600000)/60000,
		(TimeLengthIni%60000)/1000, TimeLengthIni%1000);
		
	if (delay >=0)
		StartFramesAdded= (long int)((delay+(FrameDuration/2.0))/FrameDuration);
	else
		StartFramesAdded= (long int)((delay-(FrameDuration/2.0))/FrameDuration);

	EndFramesAdded=0;
	if (same)
		EndFramesAdded= -StartFramesAdded;


//	TimeLengthEst=	TimeLengthIni + (__int64)((StartFramesAdded+EndFramesAdded)*FrameDuration);
	TimeLengthEst=	(__int64)((nuframes+StartFramesAdded+EndFramesAdded)*FrameDuration);
	sprintf (s_TimeLengthEst, "%02I64d:%02I64d:%02I64d.%03I64d",
		TimeLengthEst/(3600000), (TimeLengthEst%3600000)/60000,
		(TimeLengthEst%60000)/1000, TimeLengthEst%1000);

	NotFixedDelay= delay - StartFramesAdded*FrameDuration;
	
	if (debug) 
	{
		// fprintf (stdout,"[ac3info]\n");
		// fprintf (stdout,"delayac3 version=%s\n",version);
		// fprintf (stdout,"Bitrate=%ld\n",rate);
		// fprintf (stdout,"Fsample=%4.1lf\n",fsample);
		// fprintf (stdout,"AC3mode=%s\n",mode);
		// fprintf (stdout,"FileSize=%I64d\n", statbuf.st_size);
		// fprintf (stdout,"BytesPerFrame=%ld\n",BytesPerFrame);
		// fprintf (stdout,"TotalFrames=%I64d\n", nuframes);
		// fprintf (stdout,"FrameDuration=%3.2lf\n", FrameDuration);
		// fprintf (stdout,"OriginalDuration=%s\n", s_TimeLengthIni);
		// fprintf (stdout,"DelayToBeCorrected=%4.2lf\n", delay);
	}

	// fprintf (out,"[ac3info]\n");
	// fprintf (out,"delayac3 version=%s\n",version);
	// fprintf (out,"Bitrate=%ld\n",rate);
	// fprintf (out,"Fsample=%4.1lf\n",fsample);
	// fprintf (out,"AC3mode=%s\n",mode);
	// fprintf (out,"FileSize=%I64d\n", statbuf.st_size);
	// fprintf (out,"BytesPerFrame=%ld\n",BytesPerFrame);
	// fprintf (out,"TotalFrames=%I64d\n", nuframes);
	// fprintf (out,"FrameDuration=%3.2lf\n", FrameDuration);
	// fprintf (out,"OriginalDuration=%s\n", s_TimeLengthIni);

	if ( rest!=0 )
	{
		// fprintf (out,"Rest= Error: There are uncomplete frames\n");
		// fprintf (stdout,"Rest= Error: There are uncomplete frames\n");
		nuframes++; // to log the last frame
	}
	if (info)	// Only info, exiting without error
	{
		fclose (in);
		return "";
	}
	// fprintf (out,"\n[Fixedac3info]\n");
	// fprintf (out,"NotFixedDelay=%4.2lf\n", NotFixedDelay);
	// fprintf (out,"EstimatedFixedDuration=%s\n", s_TimeLengthEst);
	// fprintf (out,"StartFramesAdded=%ld\n", StartFramesAdded);
	// fprintf (out,"EndFramesAdded=%ld\n", EndFramesAdded);

/////////////////////////////////////////////////////////
// Translate split parameters to number of frames
/////////////////////////////////////////////////////////	
  	if (startsplit >0  )
	{
		delay2=delay-startsplit*1000.0;
		if (delay2 >=0)
			StartFramesAdded= (long int)((delay2+(FrameDuration/2.0))/FrameDuration);
		else
			StartFramesAdded= (long int)((delay2-(FrameDuration/2.0))/FrameDuration);
	}

	if (endsplit >0  )
	   EndFramesAdded-= (nuframes - (__int64)(endsplit*FramesPerSecond + 0.5));

	TimeLengthEstSplit=	(__int64)((nuframes+StartFramesAdded+EndFramesAdded)*FrameDuration);
	sprintf (s_TimeLengthEstSplit, "%02I64d:%02I64d:%02I64d.%03I64d",
		TimeLengthEstSplit/(3600000), (TimeLengthEstSplit%3600000)/60000,
		(TimeLengthEstSplit%60000)/1000, TimeLengthEstSplit%1000);

	NotFixedDelay= delay2 - StartFramesAdded*FrameDuration;

	// fprintf (out,"\n[FixedAndSplitac3info]\n");
	// fprintf (out,"NotFixedDelay=%4.2lf\n", NotFixedDelay);
	// fprintf (out,"EstimatedFixedAndSplitDuration=%s\n", s_TimeLengthEstSplit);
	// fprintf (out,"StartFramesAdded=%ld\n", StartFramesAdded);
	// fprintf (out,"EndFramesAdded=%ld\n", EndFramesAdded);


// rewind input file
	fseek (in, 0, SEEK_SET);
// open Fixed file
	if ((fix=fopen(FixedFilename,"wb"))==NULL)
	{
//		fclose  (out);
		fclose (in);
		// fprintf (stderr,"Unable to open output file: %s\n", FixedFilename);
		return "";
	}

/////////////////////////////////////////////////////////////////////////////
// Fill silence frame, If acmod or bitrate is not included, use first frame
/////////////////////////////////////////////////////////////////////////////
	for (i=0; i<nubytes; i++)
		silence[i]=caracter[i];
	p_silence=silence;
	if (acmod==2) 
	{
		switch (rate)
		{
			case 96:
				p_silence=ac3_96k2_48;
				break;
			case 112:
				p_silence=ac3_112k2_48;
				break;
			case 128:
				p_silence=ac3_128k2_48;
				break;
			case 160:
				p_silence=ac3_160k2_48;
				break;
			case 192:
				p_silence=ac3_192k2_48;
				break;
			case 224:
				p_silence=ac3_224k2_48;
				break;
			case 256:
				p_silence=ac3_256k2_48;
				break;
		}
	}
	else if (acmod ==7)
	{
		switch (rate)
		{
			case 224:
				p_silence=ac3_224k6_48;
				break;
			case 256:
				p_silence=ac3_256k6_48;
				break;
			case 320:
				p_silence=ac3_320k6_48;
				break;
			case 384:
				p_silence=ac3_384k6_48;
				break;
			case 448:
				p_silence=ac3_448k6_48;
				break;
			case 512:
				p_silence=ac3_512k6_48;
				break;
			case 640:
				p_silence=ac3_640k6_48;
				break;
		}
	}
	
	nuframeswritten=0;


/////////////////////////////////////////////////////////
// Output initial frames (If positive delay)
/////////////////////////////////////////////////////////	
	for (i=0; i< StartFramesAdded ; i++)
	{
		writeframe (fix, p_silence);
		nuframeswritten++;
	}
/////////////////////////////////////////////////////////
// Read & write frame by frame 
//	 If EndFrames<0 stop before the end
/////////////////////////////////////////////////////////	
	// fprintf (out,"\n[ProcessingLog]\n");

	bool bOK = true;

	for (i=0; i< nuframes+EndFramesAdded && i< nuframes ; i++)
	{

		if ( WaitForSingleObject(hStop, 0) != WAIT_TIMEOUT )
		{
			bOK = false;
			break;
		}

		if ( pDlg )
		{
			MSG msg;
			while (PeekMessage(&msg, NULL, 0, 0, PM_REMOVE))
			{
				if ( msg.message == WM_QUIT  )
				{
					bOK = false;
					break;
				}

				TranslateMessage(&msg);
				DispatchMessage(&msg);
			}
		}

		if ( pProgressBar )
		{
			int nPercent = (int)(((((double)i)/((double)nuframes))*100.0));
			pProgressBar->SetPos(nPercent);
		}

		f_writeframe=WF_WRITE; //indicates write frame
		nubytes=readframe (in, caracter);
		frmsizecodn=caracter[4]& 0x3F;
		BytesPerFramen=FrameSize_48[frmsizecod]*2;
		sprintf (s_time,"%02d:%02d:%02d.%03d", (int)(FrameDuration*i/3600000),
                                               ((( (int)(FrameDuration*i) )%3600000)/60000),
                                               ((( (int)(FrameDuration*i) )%60000)/1000),
                                               ((  (int)(FrameDuration*i) )%1000) );

		if ( caracter[0]!= 0x0B || caracter[1]!=0x77)
		{
			nuerrors++;
			if (maxerrorstatus==0)    maxerrorstatus=1;
			// fprintf (out, "Time %s; Frame#= %I64d. Unsynchronized frame...", s_time, i+1);
// Try to find the next sync Word and continue...
			// rewind nubytes (last frame)
			fseek(in, (long)(-1*nubytes), SEEK_CUR);
			caracter[0]=fgetc(in);
			caracter[1]=fgetc(in);
			while ((caracter[0]!= 0x0B || caracter[1]!=0x77 ) && !feof(in) )
			{
 				caracter[0]= caracter[1];
 				caracter[1]= fgetc(in);
			}
			if (caracter[0]== 0x0B && caracter[1]==0x77) 
			{
				// fprintf (out, "FIXED.\n");
			// rewind 2 bytes
				fseek(in, -2L, SEEK_CUR);
				f_writeframe=WF_SKIP; // do not write this frame
			}
			else // nothimg to do, reached end of file
			{
				// fprintf (out, "NOT FIXED.\n");
				f_writeframe=WF_SKIP; // do not write this frame
			}
		}
		else  if (nubytes < 12 || nubytes != BytesPerFramen)
		{
			nuerrors++;
			f_writeframe=WF_SKIP; // do not write this frame
			if (maxerrorstatus==0)    maxerrorstatus=1;
			// fprintf (out, "Time %s; Frame#= %I64d. Uncomplete frame. SKIPPED\n", s_time, i+1);
		}
		else
		{
// Some consistence checks
			syncwordn=   getbits (16, caracter);
			crc1n=       getbits (16, caracter);
			fscodn=      getbits (2, caracter);
			frmsizecodn= getbits (6, caracter);
			bsidn=       getbits (5, caracter);
			bsmodn=      getbits (3, caracter);
			acmodn=      getbits (3, caracter);
			if ((frmsizecodn >> 1) != (frmsizecod >>1) || fscodn != fscod ||
				bsmodn != bsmodn || acmodn != acmod)
			{
                  	nuerrors++;
                  	maxerrorstatus=2;
				// fprintf (out,"Time %s; Frame#= %I64d. Some basic parameters changed between Frame #1 and this frame\n",
//						s_time, i+1);
			}
			
// CRC calculation and fixing.

			frame_size=  FrameSize_48[frmsizecod];
			frame_size_58 = (frame_size >> 1) + (frame_size >> 3);

			cal_crc1 = ac3_crc(caracter + 4, (2 * frame_size_58) - 4, 0);
  	      	crc_inv  = pow_poly((CRC16_POLY >> 1), (16 * frame_size_58) - 16, CRC16_POLY);
	      	cal_crc1 = mul_poly(crc_inv, cal_crc1, CRC16_POLY);
	 		cal_crc2 = ac3_crc(caracter + 2 * frame_size_58, (frame_size - frame_size_58) * 2 - 2, 0);
	
			if (caracter[2]!=(cal_crc1 >> 8) ||
		    		caracter[3]!=(cal_crc1 & 0xff) )
			{
				nuerrors++;
				if (maxerrorstatus==0)    maxerrorstatus=1;
				// fprintf (out,"Time %s; Frame#= %I64d. Crc1 error %s: read = %02X%02X; calculated=%02X%02X\n",
//					s_time,i+1, s_fixcrc,
//					caracter[2], caracter[3], cal_crc1 >> 8, cal_crc1 & 0xff );
				if (fixcrc==CRC_FIX)
				{
					caracter[2]=(cal_crc1 >> 8);
					caracter[3]=cal_crc1 & 0xff;
				}
				else if (fixcrc==CRC_SKIP) f_writeframe=WF_SKIP;
				else if (fixcrc==CRC_SILENCE) f_writeframe=WF_SILENCE;
			}

			if (caracter[2*frame_size - 2]!=(cal_crc2 >> 8) ||
				caracter[2*frame_size - 1]!=(cal_crc2 & 0xff) )
			{
				nuerrors++;
				if (maxerrorstatus==0)    maxerrorstatus=1;
				// fprintf (out,"Time %s; Frame#= %I64d. Crc2 error %s: read = %02X%02X; calculated=%02X%02X\n",
//					s_time, i+1, s_fixcrc,
//					caracter[2*frame_size - 2],caracter[2*frame_size - 1],
//					cal_crc2 >> 8, cal_crc2 & 0xff );
				if (fixcrc==CRC_FIX)
				{
					caracter[2*frame_size - 2]=(cal_crc2 >> 8);
					caracter[2*frame_size - 1]=cal_crc2 & 0xff;
				}
				else if (fixcrc==CRC_SKIP) f_writeframe=WF_SKIP;
				else if (fixcrc==CRC_SILENCE) f_writeframe=WF_SILENCE;
			}
		}

//	Write frame
//	If StartFrames <0 cut (do not write) the first frames
		if (StartFramesAdded >0 || -StartFramesAdded <= i)
			if (f_writeframe==WF_WRITE)
			{
				nuframeswritten++;
				writeframe (fix, caracter);
			}
			else if (f_writeframe==WF_SILENCE)
			{
				nuframeswritten++;
				writeframe (fix, p_silence);
			}
	}
/////////////////////////////////////////////////////////
// Output ending frames (Negative delay, Positive EndFrames)
/////////////////////////////////////////////////////////	
	for (i=0; i< EndFramesAdded ; i++)
	{
		writeframe (fix, p_silence);
		nuframeswritten++;
	}
/////////////////////////////////////////////////////////
// Close and bye 
/////////////////////////////////////////////////////////	
	TimeLengthFix=	(__int64)(nuframeswritten*FrameDuration);
	sprintf (s_TimeLengthFix, "%02I64d:%02I64d:%02I64d.%03I64d",
		TimeLengthFix/(3600000), (TimeLengthFix%3600000)/60000,
		(TimeLengthFix%60000)/1000, TimeLengthFix%1000);

	// fprintf (out,"\n[ProcessResults]\n");
	// fprintf (out,"FixedDuration=%s\n", s_TimeLengthFix);
	fclose (in);
	fclose (fix);
	// fprintf (out,"Number of Errors= %d\n", nuerrors);
	// fprintf (out,"Return Value= %d\n", maxerrorstatus);
	//fclose (out);

	// Delete the original file and move the fixed one
	// Dont bother if we quit
	CString csOutFile = FixedFilename;
	if ( bOK && nBytesWritten )	// Dont overwrite with an empty file!
	{
		SetFileAttributes(Filename, 0);
		DeleteFile(Filename);

		char szFinalName[MAX_PATH];

		strcpy(szFinalName, Filename);
		if ( strrchr(szFinalName, '.') )
		{
			strrchr(szFinalName, '.')[0] = 0;
			strcat(szFinalName, "_CorrectedTo0ms.ac3");
		}

		rename(FixedFilename, szFinalName);
		csOutFile = szFinalName;
	}

	return csOutFile;
}
