#include "stdafx.h"
#include "demux.h"

#include "dprintf.h"

void CDemux::write_buf(unsigned char *buf, int size)
{
	memcpy(m_OutPtrEnd, buf, size);
	m_OutPtrEnd += size;
}

CDemux::CDemux(bool bTS, int stream_or_pid, int sub_stream)
{
	m_PTS = 0;
	m_FirstPTS = 0;
	m_BigPTS = 0;
	m_BigFirstPTS = 0;
	m_bES = false;
	m_OutBuf = NULL;
	m_OutPtrEnd = NULL;

    state = DEMUX_SKIP;
    state_bytes = 0;

	if ( bTS == false && stream_or_pid == 0 && sub_stream == 0 )
		m_bES = true;
	else
	{
		if ( bTS )		// if Transport Stream
		{
			m_demux_pid = stream_or_pid;
			m_demux_track = 0;
			m_sub_stream = 0;
		}
		else
		{
			m_demux_pid = 0;
			m_demux_track = stream_or_pid;
			m_sub_stream = sub_stream;
		}

		m_OutBuf = new unsigned char[OUTBUFFERSIZE];
		m_OutPtrEnd = m_OutBuf;

		ZeroMemory(head_buf, 264);
	}
}

CDemux::~CDemux()
{
	if ( m_OutBuf )
		delete m_OutBuf;
}

void CDemux::Reset()
{
	m_PTS = 0;
	m_FirstPTS = 0;
	m_BigPTS = 0;
	m_BigFirstPTS = 0;
	m_OutPtrEnd = m_OutBuf;

    state = DEMUX_SKIP;
    state_bytes = 0;

	ZeroMemory(head_buf, 264);
}

int CDemux::Load(unsigned char *buf, int bufsize)
{
	if ( m_bES )
	{
		m_OutBuf = buf;
		return bufsize;
	}
	else
	{
		if ( !m_demux_pid )
		{
			return PS_Load(buf, bufsize);
		}
		else
			return TS_Load(buf, bufsize);
	}
}

int CDemux::PS_Load(unsigned char *buf, int bufsize)
{
	m_OutPtrEnd = m_OutBuf;

	unsigned char * end = buf + bufsize;

	int read = (int) m_OutPtrEnd;

	if (demux (buf, end, 0))
		return 0;//DEMUX_PROGRAM_END_CODE;
	
	return (int)m_OutPtrEnd-read;
}

// All Packets must be in 188 in size
#define DEMUX_PAYLOAD_START 1
int CDemux::TS_Load(unsigned char *in_buf, int bytes)	
{
	m_OutPtrEnd = m_OutBuf;

	//#define PACKETS (BUFFER_SIZE / 188)
    unsigned char * buf;
    unsigned char * data;
    unsigned char * end;
	unsigned char * buffer = in_buf;
    int i;
    int pid;
	int packets;

	int read = (int) m_OutPtrEnd;
    //do 
	{
		//packets = fread (buffer, 188, PACKETS, in_file);
		//packets = read (in_file, buffer, 188*PACKETS);
		packets = bytes/188;

		for (i = 0; i < packets; i++) 
		{
			buf = buffer + (i * 188);
			end = buf + 188;
			if (buf[0] != 0x47) 
			{
				dprintf ("bad sync byte\n");
				
			}

			pid = ((buf[1] << 8) + buf[2]) & 0x1fff;
			
			if (pid != m_demux_pid)
				continue;
			
			data = buf + 4;
			
			if (buf[3] & 0x20) 
			{	
				data = buf + 5 + buf[4];
				if (data > end)
					continue;
			}
			
			if (buf[3] & 0x10)
			{
				demux (data, end, (buf[1] & 0x40) ? DEMUX_PAYLOAD_START : 0);
				//write_buf(data, end-data);
			}
		}

    }// while (packets == PACKETS);
	return (int)m_OutPtrEnd-read;
}

int CDemux::demux (unsigned char * buf, unsigned char * end, int flags)
{
    static int mpeg1_skip_table[16] = { 0, 0, 4, 9, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 };

    /*
     * the demuxer keeps some state between calls:
     * if "state" = DEMUX_HEADER, then "head_buf" contains the first
     *     "bytes" bytes from some header.
     * if "state" == DEMUX_DATA, then we need to copy "bytes" bytes
     *     of ES data before the next header.
     * if "state" == DEMUX_SKIP, then we need to skip "bytes" bytes
     *     of data before the next header.
     *
     * NEEDBYTES makes sure we have the requested number of bytes for a
     * header. If we dont, it copies what we have into head_buf and returns,
     * so that when we come back with more data we finish decoding this header.
     *
     * DONEBYTES updates "buf" to point after the header we just parsed.
     */

    unsigned char * header;
    int bytes;
    int len;

#define NEEDBYTES(x)						\
    do {							\
	int missing;						\
								\
	missing = (x) - bytes;					\
	if (missing > 0) {					\
	    if (header == head_buf) {				\
		if (missing <= end - buf) {			\
		    memcpy (header + bytes, buf, missing);	\
		    buf += missing;				\
		    bytes = (x);				\
		} else {					\
		    memcpy (header + bytes, buf, end - buf);	\
		    state_bytes = bytes + end - buf;		\
		    return 0;					\
		}						\
	    } else {						\
		memcpy (head_buf, header, bytes);		\
		state = DEMUX_HEADER;				\
		state_bytes = bytes;				\
		return 0;					\
	    }							\
	}							\
    } while (0)

#define DONEBYTES(x)		\
    do {			\
	if (header != head_buf)	\
	    buf = header + (x);	\
    } while (0)

    if (flags & DEMUX_PAYLOAD_START)
		goto payload_start;

    switch (state) 
	{
		case DEMUX_HEADER:
			if (state_bytes > 0) 
			{
				header = head_buf;
				bytes = state_bytes;
				goto continue_header;
			}
			break;

		case DEMUX_DATA:
			if (m_demux_pid || (state_bytes > end - buf)) 
			{
				if ( m_sub_stream )
				{
					if ( buf[0] == m_sub_stream )
						//fwrite (buf+4, end - buf - 4, 1, out_file);
						//write(out_file, buf+4, end - buf - 4);
						write_buf(buf+4, end - buf - 4);
				}
				else
					//fwrite (buf, end - buf, 1, out_file);
					//write(out_file, buf, end - buf);
					write_buf(buf, end - buf);
				state_bytes -= end - buf;
				return 0;
			}
			if ( m_sub_stream )
			{
				if ( buf[0] == m_sub_stream )
				//fwrite (buf+4, state_bytes-4, 1, out_file);
				//write(out_file, buf+4, state_bytes-4);
				write_buf(buf+4, state_bytes-4);
			}
			else
				//fwrite (buf, state_bytes, 1, out_file);
				//write(out_file, buf, state_bytes);
				write_buf(buf, state_bytes);
				buf += state_bytes;
			break;
    
		case DEMUX_SKIP:
			if (m_demux_pid || (state_bytes > end - buf)) 
			{
				state_bytes -= end - buf;
				return 0;
			}
			buf += state_bytes;
			break;
	}

    while (1) 
	{
	if (m_demux_pid) 
	{
	    state = DEMUX_SKIP;
	    return 0;
	}
    payload_start:
	header = buf;
	bytes = end - buf;
    continue_header:
	NEEDBYTES (4);
	if (header[0] || header[1] || (header[2] != 1)) {
	    if (m_demux_pid) {
		state = DEMUX_SKIP;
		return 0;
	    } else if (header != head_buf) {
		buf++;
		goto payload_start;
	    } else {
		header[0] = header[1];
		header[1] = header[2];
		header[2] = header[3];
		bytes = 3;
		goto continue_header;
	    }
	}
	if (m_demux_pid) 
	{
		// Does not necessarily have to be a video stream for TS stream
/*	    if ((header[3] >= 0xe0) && (header[3] <= 0xef))
		goto pes;
	    dprintf("bad stream id %x\n", header[3]);*/
		goto pes;
	    //exit (1);
	}
	switch (header[3]) {
	case 0xb9:	/* program end code */
	    /* DONEBYTES (4); */	// Might have to include the DONEBYTES4
	    /* break;         */
	    return 1;
	case 0xba:	/* pack header */
	    NEEDBYTES (12);
	    if ((header[4] & 0xc0) == 0x40) {	/* mpeg2 */
		NEEDBYTES (14);
		len = 14 + (header[13] & 7);
		NEEDBYTES (len);
		DONEBYTES (len);
		/* header points to the mpeg2 pack header */
	    } else if ((header[4] & 0xf0) == 0x20) {	/* mpeg1 */
		DONEBYTES (12);
		/* header points to the mpeg1 pack header */
	    } else {
		dprintf("weird pack header\n");
		//exit (1);
	    }
	    break;
	default:
	    if (header[3] == m_demux_track) {
	    pes:
		NEEDBYTES (7);
		if ((header[6] & 0xc0) == 0x80) {	/* mpeg2 */
		    NEEDBYTES (9);
		    len = 9 + header[8];
		    NEEDBYTES (len);
		    /* header points to the mpeg2 pes header */
			if (header[7] & 0x80) 
			{
				// MSB is not included, 33 bit value who at mpeg came up with that huh?
				m_PTS = (((buf[9] >> 1) << 30) |
					    (buf[10] << 22)		   |
						((buf[11] >> 1) << 15) |
					    (buf[12] << 7)		   | 
						(buf[13] >> 1));

				// Code for getting the full 33 bit PTS
				CurrentBfr = ( buf[9] << 24 ) |
							 ( buf[10] << 16 ) |
							 ( buf[11] << 8  ) |
							 ( buf[12] );

				BitsLeft   = 32;
				int code = Get_Bits(4);
				__int64 PTS_32, PTS_29, PTS_14;

				PTS_32 = Get_Bits(3);
				BitsLeft--;
				PTS_29 = Get_Bits(15);
				BitsLeft--;
				CurrentBfr = ( buf[12] << 24 ) |
							 ( buf[13] << 16 ) |
							 ( buf[14] << 8  ) |
							 ( buf[15] );
				BitsLeft   = 32;
				PTS_14 = Get_Bits(15);
				m_BigPTS = PTS_32 << 30; 
				m_BigPTS |= ( PTS_29 << 15 ); 
				m_BigPTS |= PTS_14; 

				if ( !m_FirstPTS )
				{
					m_FirstPTS = m_PTS;
					m_BigFirstPTS = m_BigPTS;
				}
		    }
		} else {	/* mpeg1 */
			int len_skip;
			unsigned char * ptsbuf;
		    len = 7;
		    while ((header-1)[len] == 0xff) {
			len++;
			NEEDBYTES (len);
			if (len > 23) {
			    dprintf("too much stuffing\n");
			    break;
			}
		    }
		    if (((header-1)[len] & 0xc0) == 0x40) {
			len += 2;
			NEEDBYTES (len);
		    }
			len_skip = len;
		    len += mpeg1_skip_table[(header - 1)[len] >> 4];
		    NEEDBYTES (len);
		    /* header points to the mpeg1 pes header */
			ptsbuf = header + len_skip;
		    if (ptsbuf[-1] & 0x20) 
			{
				m_PTS = (((ptsbuf[-1] >> 1) << 30) |
					   (ptsbuf[0] << 22) | ((ptsbuf[1] >> 1) << 15) |
					   (ptsbuf[2] << 7) | (ptsbuf[3] >> 1));

				// MPEG-1 might be different assume 32 bit for Big PTS
				m_BigPTS = m_PTS;

				if ( !m_FirstPTS )
				{
					m_FirstPTS = m_PTS;
					m_BigFirstPTS = m_BigPTS;
				}
		    }
		}
		DONEBYTES (len);
		bytes = 6 + (header[4] << 8) + header[5] - len;
		if (m_demux_pid || (bytes > end - buf)) 
		{
			if ( m_sub_stream )
			{
				if ( buf[0] == m_sub_stream )
					//fwrite (buf+4, end - buf - 4, 1, out_file);
					//write(out_file, buf+4, end - buf - 4);
					write_buf(buf+4, end - buf - 4);
			}
			else
			    //fwrite (buf, end - buf, 1, out_file);
				//write(out_file, buf, end - buf);
				write_buf(buf, end - buf);

		    state = DEMUX_DATA;
		    state_bytes = bytes - (end - buf);
		    return 0;
		} else if (bytes <= 0)
		    continue;
		if ( m_sub_stream )
		{
			if ( buf[0] == m_sub_stream )
				//fwrite (buf + 4, bytes - 4, 1, out_file);
				//write(out_file, buf + 4, bytes - 4);
				write_buf(buf + 4, bytes - 4);
		}
		else
		{
			//fwrite (buf, bytes, 1, out_file);
			//write(out_file, buf, bytes);
			write_buf(buf, bytes);
		}
		buf += bytes;
	    } else if (header[3] < 0xb9) {
		fprintf (stderr,
			 "looks like a video stream, not system stream\n");
		DONEBYTES (4);
	    } else {
		NEEDBYTES (6);
		DONEBYTES (6);
		bytes = (header[4] << 8) + header[5];
		if (bytes > end - buf) {
		    state = DEMUX_SKIP;
		    state_bytes = bytes - (end - buf);
		    return 0;
		}
		buf += bytes;
	    }
	}
    }
}

int CDemux::TestFile(const char *szFileName, bool *mpeg2)
{
#define DEMUXTESTBUF 4096
	
	int  nRet = DEMUX_FAIL;
	int  nBufSize = DEMUXTESTBUF;
	BYTE *pBuf	  = new BYTE[DEMUXTESTBUF];
	
	FILE *fin = fopen(szFileName, "rb");

	if ( fin )
	{
		nBufSize = fread(pBuf, 1, DEMUXTESTBUF, fin);
		fclose(fin);

		nRet = TestStream(pBuf, nBufSize, mpeg2);
	}

	delete [] pBuf;
	
	return nRet;
}


// Crap but will work probably with 95% of all files, MPEG-2 Only (but will probably work with MPEG-1)
int CDemux::TestStream(unsigned char *buf, int bufsize, bool *mpeg2)
{
	int nRetType = DEMUX_UNKNOWN;
	bool PS = false;
	bufsize -= 5;

	// TRANSPORT STERAM TEST
	for ( int i = 0; i != bufsize-(188*2); i++ )
	{
		// Sync byte is always 0x47 at start of 188 packet
		if ( buf[i] == 0x47 && buf[i+188] == 0x47 && buf[i+(188*2)] == 0x47 )	
			return DEMUX_TS;
	}

	// MPEG-1/2 TEST
	for ( i = 0; i != bufsize; i++ )
	{
		if ( buf[i] == 0 && buf[i+1] == 0 && buf[i+2] == 0x01 )
		{
			if ( buf[i+3] == 0xba )
			{
				PS = true;

				if ( mpeg2 )
					*mpeg2 = ((buf[i+4] & 0xc0)>>6)==1; // Get first two bits
			}
		}
	}	

	if ( PS )
		nRetType = DEMUX_PS;
	else
		nRetType = DEMUX_ES;

	return nRetType;	
}

int CDemux::OldTestStream(unsigned char *buf, int bufsize)
{
	int ps=0,es=0,ts=0;
	
	bufsize -= 4;

	for ( int i = 0; i != bufsize-(188*2); i++ )
	{
		// Sync byte is always 0x47 at start of 188 packet
		if ( buf[i] == 0x47 && buf[i+188] == 0x47 && buf[i+(188*2)] == 0x47 )	
			return DEMUX_TS;
	}

	for ( i = 0; i != bufsize; i++ )
	{
		if ( buf[i] == 0 && buf[i+1] == 0 && buf[i+2] == 0x01 )
		{
			if ( buf[i+3] == 0xba )
				ps++;
			if ( buf[i+3] == 0xb3 )
				es++;
		}
	}	

	char szTemp[100];
	sprintf(szTemp,"ES: %d PS: %d\n", es, ps);
	OutputDebugString(szTemp);
	if ( es > ps )
		return DEMUX_ES;
	if ( ps > es )
		return DEMUX_PS;

	return DEMUX_PS;	// default 
}

unsigned int CDemux::Get_Bits(unsigned int N)
{
		int Val = (CurrentBfr << (32 - BitsLeft)) >> (32 - N);
		BitsLeft -= N;
		return Val;
}
