#include "MPEGReader.h"
#include "dprintf.h"

CMPEGReader::CMPEGReader(bool bTS, int stream_or_pid, int sub_stream)
{
	if ( bTS )
		m_nBufferSize = 188*10;
	else
		m_nBufferSize = 2048;

	m_nStream = stream_or_pid;
	m_nSubStream = sub_stream;
	m_bTS = bTS;

	m_pDemux = new CDemux(m_bTS, m_nStream, m_nSubStream);
	pBuffer  = new BYTE[m_nBufferSize];
	m_Files  = new CMultipleFiles;
	ZeroMemory(pBuffer, m_nBufferSize);
	m_bForceES = false;
}

CMPEGReader::~CMPEGReader()
{
	delete m_Files;
	delete m_pDemux;
	delete [] pBuffer;
}

bool CMPEGReader::LoadFiles(CString csFileNames)
{
	m_csFileNames = csFileNames;
	return m_Files->LoadFiles(csFileNames);
}

int	CMPEGReader::SimpleBlockRead(unsigned char **buf)
{
	int nBuffer;
	int nRead;

	do
	{
		if ( (nBuffer = m_Files->Read(pBuffer, m_nBufferSize)) <= 0 )
			return -1;

		nRead = m_pDemux->Load(pBuffer, nBuffer);

	} while ( nRead == 0 );

	*buf = m_pDemux->GetPtr();
	
	return nRead;
}

int CMPEGReader::Read(unsigned char *buf, int nSize)
{
	int nRead = 0;

	if ( m_bForceES )
	{
		nRead = m_Files->Read(buf, nSize);
	}
	else
	{
		// Read from buffer first if available
		while ( nRead < nSize )
		{
			int nReadIn =  m_DemuxedBuffer.Read(buf + nRead, nSize - nRead);
			if ( !nReadIn )
				break;
			nRead += nReadIn;
		}

		while ( nRead < nSize )
		{
			int nReadIn = m_Files->Read(pBuffer, m_nBufferSize);

			if ( nReadIn <= 0 )
			{
				nRead += m_DemuxedBuffer.Read(buf + nRead, nSize - nRead);
				break;
			}

			nReadIn = m_pDemux->Load(pBuffer, nReadIn);

			if ( nReadIn > 0 )
			{
				m_DemuxedBuffer.CopyBlockIn(m_pDemux->GetPtr(), nReadIn);
				nReadIn = m_DemuxedBuffer.Read(buf + nRead, nSize - nRead);
				nRead += nReadIn;
			}
		}
	}

	return nRead;
}

bool CMPEGReader::SeekAndReset(__int64 pos)
{
	m_DemuxedBuffer.ClearBuffer();
	m_pDemux->Reset();

	return m_Files->Seek(pos);
}

__int64 CMPEGReader::GetTotalMPEGSize()
{
	return m_Files->GetTotalSize();
}

__int64 CMPEGReader::Tell()
{
	return m_Files->Tell();
}

CMultipleFiles*	CMPEGReader::GetFilesPtr()
{
	return m_Files;
}

unsigned int CMPEGReader::GetPTS()
{
	return m_pDemux->GetPTS();
}

unsigned int CMPEGReader::GetFirstPTS()
{
	return m_pDemux->GetFirstPTS();
}

unsigned __int64 CMPEGReader::GetBigPTS()
{
	return m_pDemux->GetBigPTS();
}

unsigned __int64 CMPEGReader::GetBigFirstPTS()
{
	return m_pDemux->GetBigFirstPTS();
}

void CMPEGReader::ForceES()
{
	m_bForceES = true;
}

bool CMPEGReader::GetForceES()
{
	return m_bForceES;
}
