
#include "MemoryBuffer.h"

CMemoryBuffer::CMemoryBuffer()
{
	m_nAmountInBuffer = 0;
	m_hMutex = CreateMutex(NULL, FALSE, NULL);
}

CMemoryBuffer::~CMemoryBuffer()
{
	WaitForSingleObject(m_hMutex, INFINITE);
	ReleaseMutex(m_hMutex);

	m_MemoryBuffer.clear();
	CloseHandle(m_hMutex);
}

void CMemoryBuffer::CopyBlockIn(BYTE *block, int nSize)
{
	WaitForSingleObject(m_hMutex, INFINITE);

	CMemBlock MemBlock;

	MemBlock.CopyBlockIn(block, nSize);

	m_MemoryBuffer.push_back(MemBlock);

	m_nAmountInBuffer += nSize;
	
	ReleaseMutex(m_hMutex);
}

void CMemoryBuffer::LinkBlockIn(BYTE *block, int nSize)
{
	WaitForSingleObject(m_hMutex, INFINITE);

	CMemBlock MemBlock;

	MemBlock.LinkBlockIn(block, nSize);

	m_MemoryBuffer.push_back(MemBlock);

	m_nAmountInBuffer += nSize;

	ReleaseMutex(m_hMutex);
}

int CMemoryBuffer::Read(BYTE *Buffer, int nSize)
{
	WaitForSingleObject(m_hMutex, INFINITE);

	int nRead =  0;

	while ( nRead < nSize )
	{
		if ( !m_MemoryBuffer.size() )
			break;

		int nReadIn = (m_MemoryBuffer.front()).ReadMemory(Buffer+nRead, nSize-nRead);

		m_nAmountInBuffer -= nReadIn;

		if ( nReadIn == 0 )	// Move to next block
			m_MemoryBuffer.pop_front();

		nRead += nReadIn;
	}

	ReleaseMutex(m_hMutex);

	return nRead;
}

void CMemoryBuffer::ClearBuffer()
{
	WaitForSingleObject(m_hMutex, INFINITE);

	m_MemoryBuffer.clear();
	m_nAmountInBuffer = 0;

	ReleaseMutex(m_hMutex);
}

int CMemoryBuffer::GetSize()
{
	WaitForSingleObject(m_hMutex, INFINITE);

	int nAmountInBuffer = m_nAmountInBuffer;

	ReleaseMutex(m_hMutex);

	return nAmountInBuffer;
}