#include "stdafx.h"
#include "MultipleFiles.h"

CMultipleFiles::CMultipleFiles()
{
	m_hCurrentFile	 = -1;
	m_nTotalFileSize = 0;
}

CMultipleFiles::~CMultipleFiles()
{
	if ( m_hCurrentFile != -1 )
		_close(m_hCurrentFile);
}

bool CMultipleFiles::LoadListFile(const char *szListFile)
{
	bool bRet		= false;
	FILE *ListFile  = fopen(szListFile, "rt");
	char szTempPath[500];

	if ( ListFile )
	{
		bRet = true;

		while (	fscanf(ListFile, "%s", szTempPath) )
		{
			if ( feof(ListFile) )
				break;
			if ( !AddFile(szTempPath) )
				bRet = false;
		}

		fclose(ListFile);
	}

	return bRet;
}

bool CMultipleFiles::LoadFiles(const char *szFileNames)
{
	// Check for .lst extension
	if ( strrchr(szFileNames, '.') )
		if ( stricmp(strrchr(szFileNames, '.'), ".lst") == 0 )
			return LoadListFile(szFileNames);

	bool bRet	= false;
	int  nLen	= strlen(szFileNames)+1;
	
	char szTempPath[500];
	char *p = szTempPath;

	for ( int i = 0; i != nLen; i++ )
	{
		if ( szFileNames[i] == '+' ||  szFileNames[i] == 0 )
		{
			*p = 0;
			p = szTempPath;

			// Remove any trailing or beginning spaces
			if ( p[0] == ' ' )
				p++;
			if ( p[strlen(p)-1] == ' ' )
				p[strlen(p)-1] = 0;
			
			if ( !AddFile(p) )
				break;

			if ( szFileNames[i] == 0 ) // Finished and everythings ok
				bRet = true;

			p = szTempPath;
		}
		else
			*p++ = szFileNames[i];
	}

	if ( !bRet )
	{
		// Clean up
		CleanUp();
	}

	return bRet;
}

bool CMultipleFiles::AddFile(const char *szPathName)
{
	bool bRet = false;

	if ( GetFileAttributes(szPathName) != (DWORD)-1 )
	{
		CFileData File;
		int FileH = _open(szPathName, _O_RDONLY | _O_BINARY );

		if ( FileH != -1 )
		{
			// Get File Size
			_lseeki64(FileH, 0, SEEK_END);
			File.nFileSize = _telli64(FileH);
			_close(FileH);

			// Set name and push into list
			File.csPath = szPathName;
			m_Files.push_back(File);
			
			bRet = true;

			// Open file if one isn't already
			if ( m_hCurrentFile == -1 )
			{
				m_hCurrentFile = _open(File.csPath, _O_RDONLY | _O_BINARY );
				m_nCurrentFile = 0;
			}

			m_nTotalFileSize += File.nFileSize;
		}
	}

	return bRet;
}

void CMultipleFiles::CleanUp()
{
	if ( m_hCurrentFile != -1 )
		_close(m_hCurrentFile);

	m_Files.clear();
}

void CMultipleFiles::Close()
{
	CleanUp();
}

int	CMultipleFiles::Read(unsigned char *buf, int nSize)
{
	int nRead = 0;

	// Check to make sure there is a file open
	if ( m_hCurrentFile != -1 )
	{
		while ( nRead < nSize )
		{
			int nReadIn = _read(m_hCurrentFile, buf + nRead, nSize - nRead);

			if ( nReadIn == 0 )	// Move to next file
			{
				m_nCurrentFile ++;
			
				if ( m_nCurrentFile >= m_Files.size() )
				{
					m_nCurrentFile--;
					break;
				}

				_close(m_hCurrentFile);
				m_hCurrentFile = _open(m_Files[m_nCurrentFile].csPath, _O_RDONLY | _O_BINARY );
			}
			
			nRead += nReadIn;
		}
	}
	
	return nRead;
}

bool CMultipleFiles::Seek(__int64 nPos, int nSeekMethod)
{
	bool bRet = false;

	vector<CFileData>::iterator iter;
	__int64 nCount = 0;
	
	switch ( nSeekMethod )
	{
	case SEEK_SET:
		for ( iter = m_Files.begin(); iter != m_Files.end(); iter++ )
		{
			nCount += (*iter).nFileSize;

			if ( nCount > nPos )
			{
				if ( m_hCurrentFile != -1 )
					_close(m_hCurrentFile);
				
				m_hCurrentFile = _open(iter->csPath, _O_RDONLY | _O_BINARY );
				m_nCurrentFile = iter - m_Files.begin();

				bRet = true;
				break;
			}
		}
		break;
	}

	return bRet;
}

__int64 CMultipleFiles::Tell()
{
	if ( m_hCurrentFile == -1 )
		return -1;

	__int64 nCount = 0;
	
	for ( int i = 0; i != m_nCurrentFile; i++ )
		nCount += m_Files[i].nFileSize;

	return nCount + _telli64(m_hCurrentFile);
}