#include "PMSParser.h"
#include <set>

using namespace std;

#define TS_AMOUNT_TO_READ (1024*1024*5)

class CPID
{
public:
	CPID(int nPid)
	{
		count = 1;
		pid = nPid;
	}
	
	bool operator==(const CPID &comp) const
	{
		return (pid == comp.pid);
	}

	bool operator<(const CPID &comp) const
	{
		return (pid < comp.pid);
	}
	
	int count, pid;
};

CPMSParser::CPMSParser()
{
}

bool CPMSParser::GetPIDs(const char *szFN, int ProgramID, list<CStreamInfo> *pStreamList)
{
	FILE *fin = fopen(szFN, "rb");

	if ( !fin || !pStreamList )
		return false;

	bool bRet = false;

	set<int> PidSET;
		
	unsigned char byte;
	// First find a sync byte
	while ( true )
	{
		fread(&byte, 1, 1, fin);
		if ( byte == PMS_TS_SYNC_BYTE )
		{
			fseek(fin, 187, SEEK_CUR);
			fread(&byte, 1, 1, fin);
			if ( byte == PMS_TS_SYNC_BYTE )
			{
				fseek(fin, -1, SEEK_CUR);
				break;
			}
			fseek(fin, -188, SEEK_CUR);
		}
	}


	unsigned char buf[188];

	fseek(fin, 0, SEEK_SET);

	set<CPID> Pids;

	while ( fread(buf, 1, 4, fin) == 4 )
	{

		if ( ftell(fin) >= TS_AMOUNT_TO_READ )
		{
			bRet = true;
			break;
		}

		if ( buf[0] != PMS_TS_SYNC_BYTE )
		{
			break;
		}


		int PID = 0;

		PID = (buf[1] & 0x1f) << 8;
		PID |= buf[2];

		CPID pid(PID);
		if ( Pids.find(pid) == Pids.end() )
			Pids.insert(pid);
		else
			(Pids.find(pid))->count++;


		if ( PID == ProgramID )
		{
			long currentpos = ftell(fin);
			fread(buf, 1, 1, fin);
			fseek(fin, buf[0], SEEK_CUR);
			fread(buf, 1, 12, fin);
			if ( buf[0] == 0x2 )
			{
				int sectionLength = ((buf[1] & 0x0f) << 8) | buf[2];
				int m_programNumber = (buf[3] << 8) | buf[4];
				int m_PCR_PID = ((buf[8] & 0x1f) << 8) | buf[9];

				unsigned int info_len = ((buf[10] & 0x0f) << 8) | buf[11];

				if ((info_len < 128) && (info_len > 0)) //buffer size
				{
					if ( fread(buf, 1, info_len, fin) == info_len )
					{
					}
				}
				
				int nExtra = 0;

				while ((sectionLength - 9 - 4 - info_len - nExtra) > 0)
				{
					if ( fread(buf, 1, 5, fin) == 5)
					{
						nExtra += 5;

						enum STREAM_TYPE
						{
							MPEG1_VIDEO = 0x01,
							MPEG2_VIDEO = 0x02,
							MPEG1_AUDIO = 0x03,
							MPEG2_AUDIO = 0x04,
							PRIVATE_SECTION = 0x05,
							PES_PRIVATE = 0x06,
							MHEG = 0x07,
							DSM_CC = 0x08,
							TYPE_A = 0x0A,
							TYPE_B = 0x0b,
							TYPE_C = 0x0c,
							TYPE_D = 0x0d,
							AC3_AUDIO = 129
						};

						struct ElementaryStream
						{
							int Type;
							int PID;
						} es;

						es.Type = (STREAM_TYPE)buf[0];
						es.PID = ((buf[1] & 0x1f) << 8) | buf[2];
						int tempLen = ((buf[3] & 0x0f) << 8) | buf[4];
						nExtra += tempLen;
						fseek(fin, tempLen, SEEK_CUR);
					
						CStreamInfo StreamInfo;

						if ( PidSET.find(es.PID) == PidSET.end() )
						{
							PidSET.insert(es.PID);
							switch ( es.Type )
							{
							case MPEG1_VIDEO:
								StreamInfo.nPID = es.PID;
								StreamInfo.stStreamType = STREAM_M1V;
								pStreamList->push_front(StreamInfo);
								break;
							case MPEG2_VIDEO:
								StreamInfo.nPID = es.PID;
								StreamInfo.stStreamType = STREAM_M2V;
								pStreamList->push_front(StreamInfo);
								break;
							case MPEG1_AUDIO:
							case MPEG2_AUDIO:
								StreamInfo.nPID = es.PID;
								StreamInfo.stStreamType = STREAM_MPA;
								pStreamList->push_back(StreamInfo);
								break;
							case AC3_AUDIO:
								StreamInfo.nPID = es.PID;
								StreamInfo.stStreamType = STREAM_M2V;
								pStreamList->push_back(StreamInfo);
								break;
							default:
								dprintf("Unknown PID type!!!");
								break;
							}
						}
					}
					else
						break;

				}			
			}
		
			fseek(fin, currentpos, SEEK_SET);
		}

		fseek(fin, 188-4, SEEK_CUR);
	}

	set<CPID>::iterator iter = Pids.begin();

	dprintf("PMS Found: 0x%X (%d) \n", ProgramID, ProgramID);
	for ( ; iter != Pids.end(); iter++ )
	{
		dprintf("PIDs 0x%X (%d)  Count: %d\n", (*iter).pid, (*iter).pid, (*iter).count);
	}

	fclose(fin);

	return bRet;
}