// ReJigDlg.cpp : implementation file
//

#include "stdafx.h"
#include "ReJig.h"
#include "ReJigDlg.h"
#include "RequantFunctions.h"
#include "Registry.h"
#include "Authoring.h"
#include "CreateParams.h"
#include <stdlib.h>

#ifdef USE_DEBUG_NEW
#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif
#endif
/////////////////////////////////////////////////////////////////////////////
// CReJigDlg dialog

CReJigDlg::CReJigDlg(CWnd* pParent /*=NULL*/)
	: CDialog(CReJigDlg::IDD, pParent)
{
	//{{AFX_DATA_INIT(CReJigDlg)
	m_csOutputFile = _T("");
	m_fCompressionFactor = 70.0;
	m_DelayAC3 = FALSE;
	m_NoOfPasses = 0;
	m_OutputSizeMB = 3451.0;
	//}}AFX_DATA_INIT
	// Note that LoadIcon does not require a subsequent DestroyIcon in Win32
	m_hIcon					= AfxGetApp()->LoadIcon(IDR_MAINFRAME);
	pszMPGFilterExt			= "MPEG-2/CCEData Files (*.vob;*.mpg;*.ts;*.m2v;*.txt)|*.vob;*.mpg;*.ts;*.m2v;*.txt|All Files (*.*)|*.*||";
	m_hStop					= CreateEvent(NULL, TRUE, FALSE, NULL);
	m_hBusy					= CreateEvent(NULL, TRUE, TRUE, NULL);
	m_PreviousJobSelection  = LB_ERR;
	m_bQuietMode			= false;
	m_bQuitApp				= false;
	m_bCloseDownAfter		= false;
	m_bAutoRun				= false;
}

void CReJigDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CReJigDlg)
	DDX_Control(pDX, IDC_DELAYAC3, m_DelayAC3Box);
	DDX_Control(pDX, IDC_IFOMODE, m_IFOMode);
	DDX_Control(pDX, IDC_COMPRESSCURRENT, m_RecompressCurrent);
	DDX_Control(pDX, IDC_OUTPUTFILE, m_OutputFile);
	DDX_Control(pDX, IDOK, m_OK);
	DDX_Control(pDX, IDC_DEMUX, m_Demux);
	DDX_Control(pDX, IDC_REFRESH, m_Refresh);
	DDX_Control(pDX, IDC_STOP, m_Stop);
	DDX_Control(pDX, IDC_DEL, m_Del);
	DDX_Control(pDX, IDC_CLEAR, m_Clear);
	DDX_Control(pDX, IDC_ADD, m_Add);
	DDX_Control(pDX, IDC_PROGRESS, m_ProgressBar);
	DDX_Control(pDX, IDC_STREAMLIST, m_lstStreamList);
	DDX_Control(pDX, IDC_INPUTLIST, m_lstInputList);
	DDX_Text(pDX, IDC_OUTPUTFILE, m_csOutputFile);
	DDX_Text(pDX, IDC_COMPRESSFACTOR, m_fCompressionFactor);
	DDV_MinMaxDouble(pDX, m_fCompressionFactor, 1., 100.);
	DDX_Check(pDX, IDC_DELAYAC3, m_DelayAC3);
	DDX_Radio(pDX, IDC_1PASS, m_NoOfPasses);
	DDX_Text(pDX, IDC_OUTPUTSIZE, m_OutputSizeMB);
	//}}AFX_DATA_MAP
}

BEGIN_MESSAGE_MAP(CReJigDlg, CDialog)
	//{{AFX_MSG_MAP(CReJigDlg)
	ON_WM_PAINT()
	ON_WM_QUERYDRAGICON()
	ON_BN_CLICKED(IDC_ADD, OnAdd)
	ON_BN_CLICKED(IDC_DEL, OnDel)
	ON_BN_CLICKED(IDC_CLEAR, OnClear)
	ON_WM_VKEYTOITEM()
	ON_BN_CLICKED(IDC_REFRESH, OnRefresh)
	ON_BN_CLICKED(IDC_DEMUX, OnDemux)
	ON_BN_CLICKED(IDC_STOP, OnStop)
	ON_BN_CLICKED(IDC_BROWSE, OnBrowse)
	ON_BN_CLICKED(IDOK, OnRecompressAll)
	ON_LBN_SELCHANGE(IDC_INPUTLIST, OnSelChangeInputlist)
	ON_BN_CLICKED(IDC_COMPRESSCURRENT, OnRecompressCurrent)
	ON_BN_CLICKED(IDC_QUITOUT, OnQuitOut)
	ON_BN_CLICKED(IDC_IFOMODE, OnIFOMode)
	ON_WM_RBUTTONDBLCLK()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CReJigDlg message handlers

void CReJigDlg::AddFile(CString csFile, CString csOutFile, int nPasses, int nOutputSize, double fLevel)
{
	// Check for CCE txt file
	CString csFirstFile = csFile;

	bool bMultipleFiles = false;

	int nPlus = csFile.Find('+');
	if ( nPlus != -1 )
	{
		csFirstFile = csFirstFile.Left(nPlus);
		if ( csFirstFile[csFirstFile.GetLength()-1] == ' ' )
			csFirstFile.SetAt(csFirstFile.GetLength()-1, 0);
		bMultipleFiles = true;
	}

	CString csTxt = csFile.Right(csFirstFile.GetLength()-csFirstFile.ReverseFind('.'));
	csTxt.MakeLower();
	if ( csTxt == ".txt" )
	{
		// Its a text file, treat as a CCEData or REJIGData file
		csTxt = csFirstFile;
        csTxt.MakeLower();
		if(csTxt.Find("rejigdata.txt") > -1)
         ReadInREJIGData(csFirstFile);
        else
		 ReadInCCEData(csFirstFile);
	}
	else
	{
		// Add to Job List
		CJob Job;
		Job.csInputFile = csFile;
		Job.fCompressionFactor = fLevel;
		Job.bMultipleFiles = bMultipleFiles;
		Job.nPasses = nPasses;
		Job.nMBDestSize = nOutputSize;

		AddString(csFile, Job.bMultipleFiles);
		
		// Create suggested output filename, or use given
		if ( csOutFile != "" )
		{
			Job.csOutputFile = csOutFile;
		}
		else
		{
			Job.csOutputFile = csFirstFile;
			int nDot = Job.csOutputFile.ReverseFind('.');
			if ( nDot != -1 )
			{
				Job.csOutputFile = Job.csOutputFile.Left(nDot);
			}
			Job.csOutputFile += "_ReJig.m2v";
		}

		m_JobList.push_back(Job);
	}

	// Select this one on the input list
	m_lstInputList.SetCurSel(m_lstInputList.GetCount()-1);
	m_PreviousJobSelection = m_lstInputList.GetCurSel();
	OnRefresh();
	UpdateData(FALSE);
	HorizScrollBox();
}

// For Testing
void main2();

double ui64ToDouble(unsigned __int64 ui64)
{
  __int64 i64 = (ui64 & 0x7FFFFFFFFFFFFFF);
  double dbl = (double) i64;
  if (ui64 & 0x8000000000000000)
    dbl += (double) 0x8000000000000000;
  return dbl;
}

bool CReJigDlg::SplashLoop()
{
	bool bContinue = true;
	bool bRet = false;

	while ( bContinue )
	{
		CReJigSplashDlg SplashDlg;
		CIFOModeDlg		IFOModeDlg;
		CAuthorDVDDlg	AuthorDVDDlg;
		CSettingsDlg	SettingsDlg;
		CRegCntrl		Reg;
		DWORD			nUseOldEngine;

		switch ( SplashDlg.DoModal() )
		{
		case IDOK:
			bContinue = false;
			bRet = true;
			break;
		case IDCANCEL:
			bContinue = false;
			EndDialog(0);
			break;
		case IDC_IFOMODE:
			IFOModeDlg.DoModal();
			break;
		case IDC_WIZARDMODE:
			RunWizard();
			break;
		case IDC_DVDAUTHOR:
			AuthorDVDDlg.DoModal();
			break;
		case IDC_SETTINGS:
			if ( SettingsDlg.DoModal() == IDOK )
			{
				bExternDropFrameFlag = (SettingsDlg.m_DropFrames == TRUE);
				bExternUseOldEngine  = (SettingsDlg.m_Engine == 0) ? true : false;
				bExternDeleteFiles   = (SettingsDlg.m_KeepFiles == TRUE) ? false : true;
				nUseOldEngine = (int)bExternUseOldEngine;
				Reg.WriteDWORD(HKEY_LOCAL_MACHINE, "SOFTWARE\\ReJig", "Use Old Engine", nUseOldEngine);
			}
			break;
		}
	}

	return false;
}

BOOL CReJigDlg::OnInitDialog()
{
	CDialog::OnInitDialog();

	// For Testing
	// main2();
//	unsigned __int64 i64 = 343485445324732834;
//	double a = ui64ToDouble(i64);
//	double b = i64;
//	Sub2Sup("c:\\matrix\\Sub_Stream_0xBD_0x20.sub", "c:\\matrix\\Sub_Stream_0xBD_0x20.pts", "c:\\matrix\\Sub_Stream_0xBD_0x20.new", false);

	CCreateParams SupMuxParams;
	SupMuxParams.AddParams("C:\\fugitive\\DVD\\mux.vob");
	SupMuxParams.AddParams("C:\\fugitive\\DVD\\test.vob");
	SupMuxParams.AddParams("C:\\fugitive\\Sub_Stream_0xBD_0x21.sup");
	SupMuxParams.AddParams("0x21");

	SupMux(SupMuxParams.GetArgc(), SupMuxParams.GetArgv(), NULL, NULL, m_hStop);

	// End of Testing

	// READ REGISTRY VALUE
	DWORD nUseOldEngine = 1;	// Old Engine is now Default!

	CRegCntrl Reg;

	// Use Old Engine ?
	Reg.ReadDWORD(HKEY_LOCAL_MACHINE, "SOFTWARE\\ReJig", "Use Old Engine", &nUseOldEngine);
	Reg.WriteDWORD(HKEY_LOCAL_MACHINE, "SOFTWARE\\ReJig", "Use Old Engine", nUseOldEngine);
	bExternUseOldEngine = ( nUseOldEngine == TRUE );

	// SPLASH SCREEN
	if ( __argc <= 1 )
		SplashLoop();

	// Set the icon for this dialog.  The framework does this automatically
	//  when the application's main window is not a dialog
	SetIcon(m_hIcon, TRUE);			// Set big icon
	SetIcon(m_hIcon, FALSE);		// Set small icon

	// Set Version Text
	SetWindowText(CString("ReJig v"+VERSION_NUMBER+" by Nic"));

	
	m_ProgressBar.SetRange(0, 100);

	CString csOutputFile("");

	// Command line parsing
	double fLevel = 70.0;
	int nOutputSize = 3541;
	int nPasses = 0;

	for ( int i = 1; i < (__argc); i++ )
	{
		char *p = __argv[i]+1;	// Skip the '-'

		CString csParam = p;
		csParam.MakeLower();

		if ( csParam == "o" )
		{
			i++;
			
			if ( i != __argc )
			{
				csOutputFile = __argv[i];
			}
		}

		if ( csParam == "i" )
		{
			i++;
			
			if ( i != __argc )
			{
				AddFile(__argv[i], csOutputFile, nPasses, nOutputSize, fLevel);
			}
		}

		if ( csParam == "level" )
		{
			i++;

			if ( i != __argc )
			{
				fLevel = atof(__argv[i]);
				nPasses = 0;
			}
		}

		if ( csParam == "size" )
		{
			i++;

			if ( i != __argc )
			{
				nOutputSize = atoi(__argv[i]);
				nPasses = 1;
			}
		}

		if ( csParam == "close" )
			m_bCloseDownAfter = true;
		
		if ( csParam == "quiet" )
			m_bQuietMode = true;
	
		if ( csParam == "auto" )
			m_bAutoRun = true;
	}

	if ( m_bAutoRun )
		PostMessage(WM_COMMAND, IDOK, NULL);

	return TRUE;  // return TRUE  unless you set the focus to a control
}

// If you add a minimize button to your dialog, you will need the code below
//  to draw the icon.  For MFC applications using the document/view model,
//  this is automatically done for you by the framework.

void CReJigDlg::OnPaint() 
{
	if (IsIconic())
	{
		CPaintDC dc(this); // device context for painting

		SendMessage(WM_ICONERASEBKGND, (WPARAM) dc.GetSafeHdc(), 0);

		// Center icon in client rectangle
		int cxIcon = GetSystemMetrics(SM_CXICON);
		int cyIcon = GetSystemMetrics(SM_CYICON);
		CRect rect;
		GetClientRect(&rect);
		int x = (rect.Width() - cxIcon + 1) / 2;
		int y = (rect.Height() - cyIcon + 1) / 2;

		// Draw the icon
		dc.DrawIcon(x, y, m_hIcon);
	}
	else
	{
		CDialog::OnPaint();
	}
}

// The system calls this to obtain the cursor to display while the user drags
//  the minimized window.
HCURSOR CReJigDlg::OnQueryDragIcon()
{
	return (HCURSOR) m_hIcon;
}

/////////////////////////////////////////////////////////////////////////////////////////////////
// Miscellaneous Code																	   //
/////////////////////////////////////////////////////////////////////////////////////////////////

CString CReJigDlg::FormatText(char* fmt, ...)
{
	char printString[5000];
	va_list argp;
	va_start(argp, fmt);
	vsprintf(printString, fmt, argp);
	va_end(argp);
   
	return CString(printString);
}

bool CReJigDlg::DoEvents()
{
	MSG msg;
	while (PeekMessage(&msg, NULL, 0, 0, PM_REMOVE))
	{
		if (msg.message == WM_QUIT)
			return false;

		TranslateMessage(&msg);
		DispatchMessage(&msg);
	}

	return true;
}

// bEnable is false when we want to get busy
void CReJigDlg::ButtonControl(bool bEnable)
{
	if ( bEnable )
	{
		SetEvent(m_hBusy);
	}
	else
	{
		ResetEvent(m_hBusy);
		ResetEvent(m_hStop);
	}

	m_Add.EnableWindow(bEnable);
	m_Clear.EnableWindow(bEnable);
	m_Del.EnableWindow(bEnable);
	m_Refresh.EnableWindow(bEnable);
	m_Demux.EnableWindow(bEnable);
	m_IFOMode.EnableWindow(bEnable);
	m_DelayAC3Box.EnableWindow(bEnable);
	m_OK.EnableWindow(bEnable);
	m_RecompressCurrent.EnableWindow(bEnable);
	m_Stop.EnableWindow(!bEnable);
	m_ProgressBar.SetPos(0);
	GetDlgItem(IDC_1PASS)->EnableWindow(bEnable);
	GetDlgItem(IDC_2PASS)->EnableWindow(bEnable);
	GetDlgItem(IDC_COMPRESSFACTOR)->EnableWindow(bEnable);
	GetDlgItem(IDC_OUTPUTSIZE)->EnableWindow(bEnable);

	if ( bEnable && m_bQuitApp )
		OnCancel();
}

/////////////////////////////////////////////////////////////////////////////////////////////////
// Input MPEG-2 Files List Code																	   //
/////////////////////////////////////////////////////////////////////////////////////////////////

void CReJigDlg::AddString(CString csFile, bool bMultiple)
{
	if ( !bMultiple )
	{
		if ( csFile.ReverseFind('\\') != -1 )
		{
			csFile = csFile.Right(csFile.GetLength()-csFile.ReverseFind('\\')-1);
		}
	}

	csFile = "[NOT DONE] " + csFile;

	m_lstInputList.AddString(csFile);
}

void CReJigDlg::SetState(int nIndex, bool bSuccess)
{
	if ( nIndex != LB_ERR )
	{
		CString csFile = m_JobList[nIndex].csInputFile;
		if ( !m_JobList[nIndex].bMultipleFiles )
		{
			if ( csFile.ReverseFind('\\') != -1 )
			{
				csFile = csFile.Right(csFile.GetLength()-csFile.ReverseFind('\\')-1);
			}
		}

		if ( bSuccess )
			csFile = "[SUCCESS] " + csFile;
		else
			csFile = "[FAILED] " + csFile;

		int nSel = m_lstInputList.GetCurSel();

		m_lstInputList.DeleteString(nIndex);
		m_lstInputList.InsertString(nIndex, csFile);

		if ( nSel != LB_ERR )
			m_lstInputList.SetCurSel(nSel);
	}
}

void CReJigDlg::HorizScrollBox()
{
	// Find the longest string in the list box.
	CString str;
	CSize   sz;
	int     dx=0;
	CDC*    pDC = m_lstInputList.GetDC();
	for (int i=0;i < m_lstInputList.GetCount();i++)
	{
	   m_lstInputList.GetText( i, str );
	   sz = pDC->GetTextExtent(str);

	   if (sz.cx > dx)
		  dx = sz.cx;
	}
	m_lstInputList.ReleaseDC(pDC);
	m_lstInputList.SetHorizontalExtent(dx);
}


void CReJigDlg::ReadInCCEData(LPCSTR szCCEData)
{
	FILE *fin = fopen(szCCEData, "r");

	list<CString> lstM2VFiles;
	list<CString>::iterator iter;

	if ( fin )
	{
		char szPath[500];
		char *p = szPath;
		
		bool bInQuotes = false;
		while ( true )
		{
			int c = getc(fin);
			if ( c == EOF )
				break;

			if ( c == '"' )
			{
				bInQuotes = !bInQuotes;

				if ( !bInQuotes )
				{
					// Then we have the filename
					*p = 0;

					// Convert to a .M2V
					if ( strrchr(szPath, '.') )
						strcpy(strrchr(szPath, '.'), ".m2v");

					lstM2VFiles.push_back(szPath);
					p = szPath;
				}
			}
			else
			{
				if ( bInQuotes )
				{
					*p = c;
					p++;
				}
			}
		}
		fclose(fin);
		
		for ( iter = lstM2VFiles.begin(); iter != lstM2VFiles.end(); iter++ )
		{
			// Add the Jobs in
			CJob Job;
			Job.csInputFile = *iter;
			Job.csInputFile.Replace("nopull.",""); //<- mike
			Job.fCompressionFactor = m_fCompressionFactor;
			Job.nPasses = m_NoOfPasses;
			Job.nMBDestSize = m_OutputSizeMB;
					
			// Create suggested output filename
			Job.csOutputFile = *iter;
			int nDot = Job.csOutputFile.ReverseFind('.');
			if ( nDot != -1 )
			{
				Job.csOutputFile = Job.csOutputFile.Left(nDot);
			}
						
			//Job.csOutputFile += "_ReJig.m2v";
			int nNopull = Job.csOutputFile.Find("nopull");
			if ( nNopull != -1 )
				Job.csOutputFile += ".mpv";
			else
				Job.csOutputFile += ".mpv.m2v";

			m_JobList.push_back(Job);
			AddString(*iter);

			//Save the last input before refreshing
			// mike
            m_csOutputFile = Job.csOutputFile;

			// Select this one on the input list
			m_lstInputList.SetCurSel(m_lstInputList.GetCount()-1);
			m_PreviousJobSelection = m_lstInputList.GetCurSel();
			OnRefresh();
			HorizScrollBox();
		}
	}
}

void CReJigDlg::ReadInREJIGData(LPCSTR szREJIGData)
{
 	FILE *fin = fopen(szREJIGData, "rt");
    
    if ( fin )
	{
	 CString csToken;
	 char szBuff[500];
	 char *token;
	 
	 while(fgets(szBuff,500,fin))
	 {
	 CJob Job;
	 int nDot;
	 int param_count = 1;
	 token = strtok( szBuff, "," );
     
     //defaults
	 Job.nPasses = 0;
	 Job.nMBDestSize = 0;
     Job.fCompressionFactor = 0.0;

	  while( token != NULL )
	  {
      csToken = token;
	  csToken.Replace("\"","");
      csToken.TrimLeft();
      csToken.TrimRight();

      switch (param_count)
      {
	  // m2v File Name
	  case 1:
	  	Job.csInputFile  = csToken;
	    Job.csOutputFile = csToken;
       break;
      
	  // Pulldown Method 0,1, or 2 
      case 2: 

        nDot = Job.csOutputFile.ReverseFind('.');
		if ( nDot != -1 )
		 Job.csOutputFile = Job.csOutputFile.Left(nDot);

		if ( strstr(token,"0") )
		 Job.csOutputFile += ".mpv.m2v";
		else
		if ( strstr(token,"1") )
		 Job.csOutputFile += ".mpv";		 
		else
		if ( strstr(token,"2") )
		 Job.csOutputFile += ".nopull.mpv";

	   break; 

      // Compression Factor
	  case 3: 
       Job.fCompressionFactor = atof(csToken);              

       break;

      // Desired File Size
	  case 4: 
       Job.nPasses = 1;
	   Job.nMBDestSize = atoi(csToken);              

       break;
	  }

      param_count ++;
      token = strtok( NULL, "," );
	  } // end while token

	  // Add Job
	  m_JobList.push_back(Job);
      AddString(Job.csInputFile);
       
	  m_csOutputFile = Job.csOutputFile;
      m_fCompressionFactor = Job.fCompressionFactor;
      m_NoOfPasses = Job.nPasses;
	  m_OutputSizeMB = Job.nMBDestSize;

	  m_lstInputList.SetCurSel(m_lstInputList.GetCount()-1);
	  m_PreviousJobSelection = m_lstInputList.GetCurSel();
	  OnRefresh();
	  HorizScrollBox();
	 
	 } //end while fgets  		
		
		
	fclose(fin);
	}
}


void CReJigDlg::OnAdd() 
{
	if ( !UpdateData() )
		return;

	CAddDlg dlg;

	if ( dlg.DoModal() == IDOK && dlg.GetListSize() )
	{
		// Concatenate FileNames
		CString csFileList = dlg.GetFileList();
		CString csFirstFile = dlg.GetFirstFile();

		// Save details (if any)
		if ( m_PreviousJobSelection != LB_ERR )
			SaveJob(m_PreviousJobSelection);

		// Check for CCE or REJIG txt file
		CString csTxt = csFirstFile.Right(csFirstFile.GetLength()-csFirstFile.ReverseFind('.'));
		csTxt.MakeLower();
		if ( csTxt == ".txt" )
		{
			// Its a text file, treat as a CCEData or REJIGData file
			csTxt = csFirstFile;
            csTxt.MakeLower();
			if(csTxt.Find("rejigdata.txt") > -1)
             ReadInREJIGData(csFirstFile);
            else
			 ReadInCCEData(csFirstFile);
		}
		else
		{
			// Add to Job List
			CJob Job;
			Job.csInputFile = csFileList;
			Job.fCompressionFactor = m_fCompressionFactor;
			Job.nPasses = m_NoOfPasses;
			Job.nMBDestSize = m_OutputSizeMB;
			Job.bMultipleFiles = dlg.GetListSize() > 0 ? true : false;

			AddString(csFileList, Job.bMultipleFiles);
			
			// Create suggested output filename
			Job.csOutputFile = csFirstFile;
			int nDot = Job.csOutputFile.ReverseFind('.');
			if ( nDot != -1 )
			{
				Job.csOutputFile = Job.csOutputFile.Left(nDot);
			}
			Job.csOutputFile += "_ReJig.m2v";

			m_JobList.push_back(Job);
		}

		// Select this one on the input list
		m_lstInputList.SetCurSel(m_lstInputList.GetCount()-1);
		m_PreviousJobSelection = m_lstInputList.GetCurSel();
		OnRefresh();
		UpdateData(FALSE);
		HorizScrollBox();
	}
}

void CReJigDlg::OnDel() 
{
	int nCurrent = m_lstInputList.GetCurSel();
	if ( nCurrent != LB_ERR )
	{
		m_JobList.erase(&m_JobList[nCurrent]);
		m_lstInputList.DeleteString(nCurrent);
		
		int nNewSel = nCurrent >= m_lstInputList.GetCount() ? m_lstInputList.GetCount() : nCurrent;
		m_lstInputList.SetCurSel(nNewSel);
		WriteJob(nNewSel);
		RedrawStreamList();
	}
}

void CReJigDlg::OnClear() 
{
	m_lstInputList.ResetContent();
	m_JobList.clear();
	RedrawStreamList();
}

int CReJigDlg::OnVKeyToItem(UINT nKey, CListBox* pListBox, UINT nIndex) 
{
	if ( pListBox == &m_lstInputList && nKey == VK_DELETE )
		OnDel();

	return CDialog::OnVKeyToItem(nKey, pListBox, nIndex);
}

bool CReJigDlg::SaveJob(int nJob)
{
	if ( !UpdateData() )
		return false;

	if ( nJob != LB_ERR && m_JobList.size() > nJob )
	{
		m_JobList[nJob].fCompressionFactor = m_fCompressionFactor;
		m_JobList[nJob].nCurrentStream = m_lstStreamList.GetCurSel();
		m_JobList[nJob].csOutputFile = m_csOutputFile;
		m_JobList[nJob].nPasses = m_NoOfPasses;
		m_JobList[nJob].nMBDestSize = m_OutputSizeMB;
	}
	UpdateData(FALSE);

	return true;
}

bool CReJigDlg::WriteJob(int nJob)
{
	if ( !UpdateData() )
		return false;

	if ( nJob != LB_ERR && m_JobList.size() )
	{
		m_NoOfPasses = m_JobList[nJob].nPasses;
		m_OutputSizeMB = m_JobList[nJob].nMBDestSize;
		m_fCompressionFactor = m_JobList[nJob].fCompressionFactor;
		m_csOutputFile = m_JobList[nJob].csOutputFile;
		if ( m_JobList[nJob].nCurrentStream == LB_ERR )
			m_JobList[nJob].nCurrentStream = 0;
		m_lstStreamList.SetCurSel(m_JobList[nJob].nCurrentStream);
		
		RedrawStreamList();
	}
	UpdateData(FALSE);

	return true;
}

void CReJigDlg::OnSelChangeInputlist() 
{
	if ( !UpdateData(TRUE) )
	{
		m_lstInputList.SetCurSel(m_PreviousJobSelection);
		return;
	}
	
	int nCurrentJob = m_lstInputList.GetCurSel();

	if ( nCurrentJob != LB_ERR )
	{
		// Save details


		if ( m_PreviousJobSelection != LB_ERR )
			SaveJob(m_PreviousJobSelection);
		
		RedrawStreamList();
		
		// Write out settings
		WriteJob(nCurrentJob);

		m_PreviousJobSelection = m_lstInputList.GetCurSel();
	}

	UpdateData(FALSE);
}

/////////////////////////////////////////////////////////////////////////////////////////////////
// Input Stream List Code																	   //
/////////////////////////////////////////////////////////////////////////////////////////////////

void CReJigDlg::RedrawStreamList()
{
	m_lstStreamList.ResetContent();

	int nCurrentJob = m_lstInputList.GetCurSel();

	if ( nCurrentJob != LB_ERR )
	{
		m_JobList[nCurrentJob].GotoStreamListStart();

		CStreamInfo StreamInfo;

		while ( m_JobList[nCurrentJob].GetNextStream(&StreamInfo) )
		{
			CString csStream = StreamInfo.GetText();
			m_lstStreamList.AddString(csStream);
		}

		if ( m_JobList[nCurrentJob].nCurrentStream == LB_ERR )
			m_JobList[nCurrentJob].nCurrentStream = 0;

		m_lstStreamList.SetCurSel(m_JobList[nCurrentJob].nCurrentStream);
	}
}

void CReJigDlg::OnRefresh() 
{
	CWaitCursor WaitCursor;

	// Need to get the streams contained in the first file		
	if ( m_lstInputList.GetCount() )
	{
		CString			csInputFile;
		CStreamInfo		StreamInfo;

		int nCurrentJob = m_lstInputList.GetCurSel();

		csInputFile = m_JobList[nCurrentJob].csInputFile;
		
		// Load into MultipleFiles so to parse any multi file input
		CMultipleFiles MultipleFiles;
		MultipleFiles.LoadFiles(csInputFile);
	
		if ( MultipleFiles.GetNumberOfFiles() )
		{
			m_FileProperties.GetInfoFromFile(MultipleFiles.GetFileName(0));
			m_FileProperties.ResetIter();
			
			m_JobList[nCurrentJob].StreamInfoVector.clear();
			m_JobList[nCurrentJob].FileType = m_FileProperties.GetFileType();
			while ( m_FileProperties.GetNextStream(&StreamInfo) )
			{
				// Add to the JobList
				m_JobList[nCurrentJob].StreamInfoVector.push_back(StreamInfo);
			}

			RedrawStreamList();
			WriteJob(nCurrentJob);
		}
	}
}

void CReJigDlg::OnDemux() 
{
	if ( !UpdateData() )
		return;

	int InputListSel  = m_lstInputList.GetCurSel();
	int StreamListSel = m_lstStreamList.GetCurSel();

	if ( InputListSel != LB_ERR && StreamListSel != LB_ERR )
	{
		CStreamInfo StreamInfo = m_JobList[InputListSel].StreamInfoVector[StreamListSel];
		CString csDefExt;
		CString csFilter;
		switch ( StreamInfo.stStreamType )
		{
		case STREAM_M2V:
			csDefExt = "m2v";
			csFilter = "MPEG-2 ES (*.m2v)|*.m2v";
			break;
		case STREAM_AC3:
			csDefExt = "ac3";
			csFilter = "AC3 (*.ac3)|*.ac3";
			break;
		case STREAM_LPCM:
			csDefExt = "lpcm";
			csFilter = "LPCM (*.pcm)|*.pcm";
			break;
		case STREAM_MPA:
			csDefExt = "mpa";
			csFilter = "MPA (*.mpa)|*.mpa";
			break;
		case STREAM_DTS:
			csDefExt = "dts";
			csFilter = "DTS (*.dts)|*.dts";
			break;
		}
		csFilter += "|All Files (*.*)|*.*||";


		CFileDialog dlg(FALSE, csDefExt, NULL, OFN_HIDEREADONLY | OFN_OVERWRITEPROMPT, csFilter);
		if ( dlg.DoModal() == IDOK )
		{
			CString csFilesCat;
			csFilesCat = m_JobList[InputListSel].csInputFile;

			BYTE szBuf[4096];

			CMPEGReader *pMPEGReader = NULL;

			if ( StreamInfo.nPID )
				pMPEGReader = new CMPEGReader(true, StreamInfo.nPID);
			else
				pMPEGReader = new CMPEGReader(false, StreamInfo.nStreamNo, StreamInfo.nSubStream);

			pMPEGReader->LoadFiles(csFilesCat);
			FILE *fout = fopen(dlg.GetPathName(), "wb");

			double TotalFileSize = (double)pMPEGReader->GetTotalMPEGSize();

			if ( fout )
			{
				ButtonControl(false);

				int nDemuxed;
				while ( (nDemuxed = pMPEGReader->Read(szBuf, 4096)) > 0 )
				{
					fwrite(szBuf, 1, nDemuxed, fout);
					if ( !DoEvents() )
						break;

					if ( WaitForSingleObject(m_hStop, 0) != WAIT_TIMEOUT )
						break;

					// Set ProgressBar
					int nPercent = (int)((((double)pMPEGReader->Tell())/TotalFileSize)*100.0);
					m_ProgressBar.SetPos(nPercent);
				}

				fclose(fout);				

				if ( m_DelayAC3 && StreamInfo.stStreamType == STREAM_AC3 )
					DelayAC3(dlg.GetPathName(), (double)StreamInfo.nAudioDelay, &m_ProgressBar, m_hStop, this);

				ButtonControl(true);
			}

			delete pMPEGReader;
		}
	}
}

void CReJigDlg::OnCancel() 
{
	if ( WaitForSingleObject(m_hBusy, 0) == WAIT_TIMEOUT )
	{
		m_bQuitApp = true;
		SetEvent(m_hStop);
	}
	else
	{
		ShowWindow(SW_HIDE);
		if ( __argc > 1 || SplashLoop() )
		{
			// Quit!
			CloseHandle(m_hBusy);
			CloseHandle(m_hStop);
			CDialog::OnCancel();
		}
		else
			ShowWindow(SW_SHOW);
	}
}

void CReJigDlg::OnStop() 
{
	SetEvent(m_hStop);	
}

void CReJigDlg::OnBrowse() 
{
	CFileDialog dlg(FALSE, "m2v", NULL, OFN_HIDEREADONLY | OFN_OVERWRITEPROMPT, "M2V Files (*.m2v)|*.m2v|All Files (*.*)|*.*||");

	if ( dlg.DoModal() == IDOK )
		m_OutputFile.SetWindowText(dlg.GetPathName());
}

void CReJigDlg::Recompress(bool bAll)
{
	int nFirst = 0;

	if ( !bAll )
		nFirst = m_lstInputList.GetCurSel();

	if ( nFirst != LB_ERR )
	{
		ButtonControl(false);

		for ( int i = nFirst; i != m_JobList.size(); i++ )
		{
			m_lstInputList.SetCurSel(i);
			WriteJob(i);

			CMPEGReader *pMPEGReader = NULL;

			CStreamInfo StreamInfo;
			
			if ( m_JobList[i].StreamInfoVector.size() )
				StreamInfo = m_JobList[i].StreamInfoVector[m_JobList[i].nCurrentStream];

			// Just here for testing
			CJob job = m_JobList[i];

			if ( m_JobList[i].FileType == DEMUX_ES || StreamInfo.stStreamType == STREAM_M2V || StreamInfo.stStreamType == STREAM_M1V )
			{
				if ( m_JobList[i].FileType != DEMUX_ES && m_JobList[i].nCurrentStream != LB_ERR )
				{
					if ( StreamInfo.nPID )
						pMPEGReader = new CMPEGReader(true, StreamInfo.nPID);
					else
						pMPEGReader = new CMPEGReader(false, StreamInfo.nStreamNo, StreamInfo.nSubStream);
				}
				else
				{
					pMPEGReader = new CMPEGReader(false, 0xe0);
					pMPEGReader->ForceES();
				}

				pMPEGReader->LoadFiles(m_JobList[i].csInputFile);

				bool bCanProceed = true;

				// Check the file can be overwritten when not in quiet mode
				if ( !m_bQuietMode )
				{
					bCanProceed = false;	// Assume worse

					if ( GetFileAttributes(m_JobList[i].csOutputFile) != -1L )
					{
						CString csMsg = "File:\r\n" + m_JobList[i].csOutputFile + "\r\nAlready Exists. OverWrite?";
						if ( MessageBox(csMsg, "Overwrite Warning", MB_YESNO | MB_ICONEXCLAMATION) == IDYES )
						{
							SetFileAttributes(m_JobList[i].csOutputFile, 0);	// Allows deletion

							// Remove the file before we begin
							while ( true )
							{
								if ( !DeleteFile(m_JobList[i].csOutputFile) )
								{
									DWORD dwErr = GetLastError();
									if ( dwErr != 2 )
									{
										csMsg = "File:\r\n" + m_JobList[i].csOutputFile + "\r\nCannot be overwritten! Retry?";
										if ( MessageBox(csMsg, "Overwrite Warning", MB_YESNO | MB_ICONEXCLAMATION) == IDNO )
											break;
									}
									else
									{
										// File has disappeared.
										bCanProceed = true;		// File was deleted successfully
										break;
									}
								}
								else
								{
									bCanProceed = true;		// File was deleted successfully
									break;
								}
							}

						}
					}
					else
						bCanProceed = true;		// File doesn't exist
				}
				else
				{
					// Remove any file that might be there
					if ( GetFileAttributes(m_JobList[i].csOutputFile) != -1L )
					{
						SetFileAttributes(m_JobList[i].csOutputFile, 0);	// Allows deletion
						if ( !DeleteFile(m_JobList[i].csOutputFile) )
							bCanProceed = false;
					}
				}

				if ( bCanProceed )
				{
					if ( m_NoOfPasses == 0 ) // 1 pass encode
					{
						if ( pMPEGReader->GetForceES() )
						{
							// The ES Stream can use new compression code
							OriginalM2VSize = pMPEGReader->GetTotalMPEGSize();
							bool bRet;

							if ( bExternUseOldEngine )
								bRet = ReEncodeOld(m_JobList[i].fCompressionFactor, m_JobList[i].csOutputFile, pMPEGReader, &m_ProgressBar, m_hStop, this);
							else
								bRet = ReEncode(m_JobList[i].fCompressionFactor, m_JobList[i].csOutputFile, pMPEGReader, &m_ProgressBar, m_hStop, this);


							if ( bRet )
							{
								SetState(i, true);
							}
							else
								SetState(i, false);
						}
						else
						{
							// Old compression code for Vobs/etc
							if ( ReEncodeOld(m_JobList[i].fCompressionFactor, m_JobList[i].csOutputFile, pMPEGReader, &m_ProgressBar, m_hStop, this) )
							{
								SetState(i, true);
							}
							else
								SetState(i, false);
						}
					}
					else	// 2 pass encode
					{
						unsigned __int64 nSize = 0;
						unsigned __int64 nAdd = 0;
						unsigned __int64 TotalFileSize = pMPEGReader->GetTotalMPEGSize();

						if ( pMPEGReader->GetForceES() == false )
						{
							int nCount = 0;
							BYTE *buf = new BYTE[1048576];
							while ( (nAdd = pMPEGReader->Read(buf, 1048576)) > 0 )
							{
								nSize += nAdd;

								if ( !((nCount++)%15) )
								{
									if ( WaitForSingleObject(m_hStop, 0) != WAIT_TIMEOUT )
										break;
									int nPercent = (int)((((double)pMPEGReader->Tell())/TotalFileSize)*100.0);
									m_ProgressBar.SetPos(nPercent);
									DoEvents();
								}
							}
							delete [] buf;
						}
						else
						{
							// No need to count it, we already know!
							nSize = TotalFileSize;
						}

						if ( WaitForSingleObject(m_hStop, 0) == WAIT_TIMEOUT )
						{
							// Calculate new Percentage
							pMPEGReader->SeekAndReset(0);
							unsigned __int64 nOutSize = nSize; 
							unsigned __int64 nDestSize = (m_OutputSizeMB * 1024.0 * 1024.0);
							double fNewPercent = (((double)nDestSize)/((double)nOutSize)) * 100.0;
							dprintf("Orig Size: %I64u Output Destination Size: %I64u New Percent: %f\n", nOutSize, nDestSize, (float)fNewPercent);
							OriginalM2VSize = nOutSize;

							bool bRet;

							if ( bExternUseOldEngine )
								bRet = ReEncodeOld(fNewPercent, m_JobList[i].csOutputFile, pMPEGReader, &m_ProgressBar, m_hStop, this);
							else
								bRet = ReEncode(fNewPercent, m_JobList[i].csOutputFile, pMPEGReader, &m_ProgressBar, m_hStop, this);
				
							if ( bRet )
							{
								SetState(i, true);
								unsigned __int64 nOutSize = ReturnOutputSize();
							}
							else
								SetState(i, false);
						}
						else
							SetState(i, false);
					}
				}
				else
					SetState(i, false);

				delete pMPEGReader;
			}

			// Only do first
			if ( !bAll )
				break;
		}

		if ( m_bCloseDownAfter )
			OnCancel();

		ButtonControl(true);
	}
}

void CReJigDlg::OnRecompressAll() 
{
	// Make sure any changes are stored in the JobList
	if ( SaveJob(m_lstInputList.GetCurSel()) )
		Recompress(true);
}


void CReJigDlg::OnRecompressCurrent() 
{
	int nCurrent = m_lstInputList.GetCurSel();

	if ( nCurrent != LB_ERR )
	{
		if ( SaveJob(nCurrent) )
			Recompress(false);		
	}
}

void CReJigDlg::OnQuitOut() 
{
	OnCancel();
}

void CReJigDlg::OnIFOMode() 
{
	CIFOModeDlg	IFOModeDlg;

	IFOModeDlg.DoModal();
}

void CReJigDlg::OnRButtonDblClk(UINT nFlags, CPoint point) 
{
	// Shhh! Secret.
//	m_IFOMode.EnableWindow(TRUE);

	CCreateParams *CreateParams = new CCreateParams;
	CreateParams->AddParams("-f");
	CreateParams->AddParams("8");
	CreateParams->AddParams("-o");
	CreateParams->AddParams("d:\\out2.mpg");
	CreateParams->AddParams("D:\\Program Files\\MSDN\\2003OCT\\DVD\\vid");
	CreateParams->AddParams("D:\\Program Files\\MSDN\\2003OCT\\DVD\\AC3_Audio_0xBD_0x80_Delay_42ms_CorrectedTo0ms.ac3");

	ButtonControl(false);
	MPlex(CreateParams->GetArgc(), CreateParams->GetArgv(), m_ProgressBar.GetSafeHwnd(), this, m_hStop);
	ButtonControl(true);

	CDialog::OnRButtonDblClk(nFlags, point);
}

////////////////////////////////////////////////////////////////////////////
// THE WIZARD															  //
////////////////////////////////////////////////////////////////////////////



void CReJigDlg::AddTitle(CString csIFOFile)
{
	// Final check for file
	if ( GetFileAttributes(csIFOFile) == -1L )
		return;

	// gotta parse the ifo, then add everything
	CTitle Title;
	AuthDrive(csIFOFile);	// Authenticate First
	if ( GetIFOInfo(csIFOFile, NULL, &Title) )
	{
		Title.csIFOFile = csIFOFile;
		Title.AutoAddVobs();

		m_Titles.push_back(Title);
		
		for ( int i = 0; i != m_Titles.back().PGCLst.size(); i++ )
		{
			CString csPGCName;
			csPGCName.Format("PGC %d [%02u:%02u:%02u]", m_Titles.back().PGCLst[i].nPGC, m_Titles.back().PGCLst[i].tLength[0], m_Titles.back().PGCLst[i].tLength[1], m_Titles.back().PGCLst[i].tLength[2]);

			if ( m_Titles.back().PGCLst[i].GotTotalSeconds() > m_nPGCTime )
			{
				m_nLongestTitle = m_Titles.size()-1;
				m_nLongestPGC   = i;
				m_nPGCTime = m_Titles.back().PGCLst[i].GotTotalSeconds();
				m_csIFOName = csIFOFile;
			}
		}
	}
	else
		MessageBox("Failed to parse IFO!", "IFO Parse Fail", MB_OK | MB_ICONEXCLAMATION);

}

bool CReJigDlg::RunWizard()
{
	bool bRet = false;
	CWizardOpenDlg			OpenDlg;
	CWizardStreamSelectDlg	SelectDlg;
	CWizardOutputSelectDlg	OutputDlg;
	CWizardEncodeDlg		EncodeDlg;

	m_nLongestTitle = -1;
	m_nLongestPGC = -1;
	m_nPGCTime = 0;

	if ( OpenDlg.DoModal() == IDOK )
	{
		if ( OpenDlg.m_OpenFrom == 0 && OpenDlg.bFoundDisc )	// 0 = DVD Drive
		{
			char szIFO[] = "X:\\VIDEO_TS\\VTS_";
			szIFO[0] = OpenDlg.lstDriveLetter[OpenDlg.nSelected];

			int nNum = 1;
			while ( true )
			{
				CString csIFOFile;
				csIFOFile.Format("%s%02d_0.IFO", szIFO, nNum++);

				if ( GetFileAttributes(csIFOFile) != -1L )
				{
					AddTitle(csIFOFile);
				}
				else
					break;
			}
		}
		else
		{
			// Load From Directory 
			for ( int nNum = 1; nNum != 30; nNum++ )
			{
				CString csFile;
				csFile.Format("%s\\VTS_%02d_0.IFO", OpenDlg.m_csInputDirectory, nNum);
				AddTitle(csFile);
			}
		}

		// Make sure somethings selected
		if ( !(m_nLongestTitle == -1 && m_nLongestPGC == -1) )
		{
			dprintf("Selecting Title: %d PGC: %d", m_nLongestTitle, m_nLongestPGC);
				
			SelectDlg.m_Titles = m_Titles;
			SelectDlg.m_nLongestPGC = m_nLongestPGC;
			SelectDlg.m_nLongestTitle = m_nLongestTitle;

			// Stream Selection
			if ( SelectDlg.DoModal() == IDOK )
			{
				// Output Directory
				if ( OutputDlg.DoModal() == IDOK )
				{
					// Start doing the Encoding
					EncodeDlg.m_Titles = m_Titles;
					EncodeDlg.nCurrentPGC = m_nLongestPGC;
					EncodeDlg.nCurrent = m_nLongestTitle;
					EncodeDlg.m_Selected = SelectDlg.m_Selected;
					EncodeDlg.m_csDirectory = OutputDlg.m_OutputDirectory;
					EncodeDlg.nOutputSize = OutputDlg.nOutputSize;
					EncodeDlg.m_csIFOName = m_csIFOName;
					if ( EncodeDlg.DoModal() == IDOK )
						bRet = true;
				}
			}
		}
	}

	return bRet;
}