#pragma once

#include "stdafx.h"
#include "MPEGReader.h"
#include "VStripSuperReader.h"
#include "dprintf.h"

#ifdef _DEBUG
#define nic_assert(x) if ( (x) == FALSE ) { dprintf("nic_assert: "); dprintf(#x); dprintf("\n"); }
#else
#define nic_assert(x)
#endif

// user defined types
typedef unsigned int		uint;
typedef unsigned char		uint8;
typedef unsigned short		uint16;
typedef unsigned int		uint32;
typedef unsigned __int64	uint64;

// RB
typedef unsigned __int8		uint8_t;
typedef signed __int8		int8_t;
typedef unsigned __int32	uint32_t;
typedef signed __int32		int32_t;

typedef char				int8;
typedef short				int16;
typedef int					int32;
typedef __int64				int64;

typedef signed int			sint;
typedef signed char			sint8;
typedef signed short		sint16;
typedef signed int			sint32;
typedef signed __int64		sint64;

#define COPY(x)\
		nic_assert(x > 0); \
		nic_assert(wbuf + x < owbuf + BUF_SIZE); \
		nic_assert(cbuf + x < orbuf + BUF_SIZE); \
		nic_assert(cbuf + x >= orbuf); \
		nic_assert(wbuf + x >= orbuf); \
		memcpy(wbuf, cbuf, x);\
		cbuf += x; \
		wbuf += x;

// useful constants
#define I_TYPE 1
#define P_TYPE 2
#define B_TYPE 3

#define REMOVE_BYTE_STUFFING	// removes 0x 00 00 00 00 00 00 used in cbr streams (look for 6 0x00 and remove 1 0x00)
								/*	4 0x00 might be legit, for exemple:
									00 00 01 b5 14 82 00 01 00 00 00 00 01 b8 .. ..
												 these two: -- -- are part of the seq. header ext.
									AFAIK 5 0x00 should never happen except for byte stuffing but to be safe look for 6 */
#define USE_FD // use 2 lasts args for input/output paths
//#define DEMO // demo mode

#define BITS_IN_BUF (8)

// 1024.0*128.0
#define REACT_DELAY (131072.0)

#ifdef USE_FD

	#define WRITE \
		mloka1 = wbuf - owbuf; \
		if ( ofd ) fwrite(owbuf, mloka1, 1, ofd); \
		outbytecnt += mloka1; \
		wbuf = owbuf; \
		mloka1 = rbuf - cbuf; if (mloka1) memmove(orbuf, cbuf, mloka1);\
		cbuf = rbuf = orbuf; rbuf += mloka1;
	
	#define LOCK(x) \
		while (unlikely(x > (rbuf-cbuf))) \
		{ \
			nic_assert(rbuf + MIN_READ < orbuf + BUF_SIZE); \
			mloka1 = ifd->Read(rbuf, MIN_READ); \
			if (!mloka1) { retVal = true; RETURN } \
			inbytecnt += mloka1; \
			rbuf += mloka1; \
		}
	
#else

	#define WRITE \
		mloka1 = wbuf - owbuf; \
		// RB
		_write(1, owbuf, mloka1); \
		outbytecnt += mloka1; \
		wbuf = owbuf; \
		mloka1 = rbuf - cbuf; if (mloka1) memmove(orbuf, cbuf, mloka1);\
		cbuf = rbuf = orbuf; rbuf += mloka1;
	
	#define LOCK(x) \
		while (unlikely(x > (rbuf-cbuf))) \
		{ \
			nic_assert(rbuf + MIN_READ < orbuf + BUF_SIZE); \
			// RB
			mloka1 = _read(0, rbuf, MIN_READ); \
			if (!mloka1) { RETURN } \
			inbytecnt += mloka1; \
			rbuf += mloka1; \
		}

#endif

// mpeg2 state
	// seq header
	static uint horizontal_size_value;
	static uint vertical_size_value;
	
	// pic header
	static uint picture_coding_type;
	
	// pic code ext
	static uint f_code[2][2];
	static uint intra_dc_precision;
	static uint picture_structure;
	static uint frame_pred_frame_dct;
	static uint concealment_motion_vectors;
	static uint q_scale_type;
	static uint intra_vlc_format;
	static uint alternate_scan;
	
	// slice or mb
	// quantizer_scale_code
	static uint quantizer_scale;
	static uint new_quantizer_scale;
	static uint last_coded_scale;
	static int	 h_offset, v_offset;
	
	// mb
	static double quant_corr;
	
	// block data
	typedef struct
	{
		uint8 run;
		short level;
	} RunLevel;

	static RunLevel block[6][65]; // terminated by level = 0, so we need 64+1
// end mpeg2 state

// global variables
static uint8	*cbuf, *rbuf, *wbuf, *orbuf, *owbuf;
static int		inbitcnt, outbitcnt;
static uint32	inbitbuf, outbitbuf;
uint64	inbytecnt, outbytecnt;
static float	fact_x;
static float	percentage_x;
static int		mloka1;
// RB
static int64	ifsize = 0;
static time_t	starttime;

#ifdef USE_FD
static CVStripSuperReader *ifd;
static FILE *ofd;
#endif

#ifdef STAT
static uint64 ori_i, ori_p, ori_b;
static uint64 new_i, new_p, new_b;
static uint64 cnt_i, cnt_p, cnt_b;
static uint64 cnt_p_i, cnt_p_ni;
static uint64 cnt_b_i, cnt_b_ni;
#endif

#ifdef DEMO
int gopCount;
#endif

#define BUF_SIZE (16*1024*1024)
#define MIN_READ (1*1024*1024)
#define MIN_WRITE (1*1024*1024)

// gcc
#ifdef HAVE_BUILTIN_EXPECT
#define likely(x) __builtin_expect ((x) != 0, 1)
#define unlikely(x) __builtin_expect ((x) != 0, 0)
#else
#define likely(x) (x)
#define unlikely(x) (x)
#endif

#define SEEKR(x)\
		cbuf += x; \
		nic_assert (cbuf <= rbuf); \
		nic_assert (cbuf < orbuf + BUF_SIZE); \
		nic_assert (cbuf >= orbuf);
	
#define SEEKW(x)\
		wbuf += x; \
		nic_assert (wbuf < owbuf + BUF_SIZE); \
		nic_assert (wbuf >= owbuf);

#ifdef STAT

	#define RETURN \
		mloka1 = rbuf - cbuf;\
		COPY(mloka1);\
		WRITE \
		free(orbuf); \
		free(owbuf); \
		\
		LOG("Stats:\n");\
		\
		LOGF("Wanted fact_x: %.1f\n", fact_x);\
		\
		LOGF("cnt_i: %.0f ", (float)cnt_i); \
		if (cnt_i) LOGF("ori_i: %.0f new_i: %.0f fact_i: %.1f\n", (float)ori_i, (float)new_i, (float)ori_i/(float)new_i); \
		else LOG("\n");\
		\
		LOGF("cnt_p: %.0f ", (float)cnt_p); \
		if (cnt_p) LOGF("ori_p: %.0f new_p: %.0f fact_p: %.1f cnt_p_i: %.0f cnt_p_ni: %.0f propor: %.1f i\n", \
			(float)ori_p, (float)new_p, (float)ori_p/(float)new_p, (float)cnt_p_i, (float)cnt_p_ni, (float)cnt_p_i/((float)cnt_p_i+(float)cnt_p_ni)); \
		else LOG("\n");\
		\
		LOGF("cnt_b: %.0f ", (float)cnt_b); \
		if (cnt_b) LOGF("ori_b: %.0f new_b: %.0f fact_b: %.1f cnt_b_i: %.0f cnt_b_ni: %.0f propor: %.1f i\n", \
			(float)ori_b, (float)new_b, (float)ori_b/(float)new_b, (float)cnt_b_i, (float)cnt_b_ni, (float)cnt_b_i/((float)cnt_b_i+(float)cnt_b_ni)); \
		else LOG("\n");\
		\
		LOGF("Final fact_x: %.1f\n", (float)inbytecnt/(float)outbytecnt);\
		exit(0);

#else

//Nick
static bool retVal;

//RB
	#define RETURN \
		mloka1 = rbuf - cbuf;\
		COPY(mloka1);\
		WRITE \
		if ( orbuf ) \
		{ \
		free(orbuf); \
		orbuf = NULL; \
		} \
		if ( owbuf ) \
		{ \
		free(owbuf); \
		owbuf = NULL; \
		} \
		if ( ofd ) fclose(ofd); \
		return retVal;
	
#endif


double ui64ToDouble(unsigned __int64 ui64);
static inline void putbits(uint val, int n)
{
	nic_assert(n < 32);
	nic_assert(!(val & (0xffffffffU << n)));

	while (unlikely(n >= outbitcnt))
	{
		wbuf[0] = (outbitbuf << outbitcnt ) | (val >> (n - outbitcnt));
		SEEKW(1);
		n -= outbitcnt;
		outbitbuf = 0;
		val &= ~(0xffffffffU << n);
		outbitcnt = BITS_IN_BUF;
	}
	
	if (likely(n))
	{
		outbitbuf = (outbitbuf << n) | val;
		outbitcnt -= n;
	}
	
	nic_assert(outbitcnt > 0);
	nic_assert(outbitcnt <= BITS_IN_BUF);
}


static inline void flush_read_buffer()
{
	int i = inbitcnt & 0x7;
	if (i)
	{
		//uint val = Show_Bits(i);
		//putbits(val, i);
		nic_assert(((unsigned int)inbitbuf) >> (32 - i) == 0);
		inbitbuf <<= i;
		inbitcnt -= i;
	}
	SEEKR(-1 * (inbitcnt >> 3));
	inbitcnt = 0;
}

static inline void flush_write_buffer()
{
	if (outbitcnt != 8) putbits(0, outbitcnt);
}

static inline void Refill_bits(void)
{
	nic_assert((rbuf - cbuf) >= 1);
	inbitbuf |= cbuf[0] << (24 - inbitcnt);
	inbitcnt += 8;
	SEEKR(1)
}

