#pragma once
#include "stdafx.h"
#include <io.h>
#include <fcntl.h>
#include "sub2sup.h"
#include "Wav.h"
#include <set>

extern unsigned __int64	VStripTotalDemuxed;
extern unsigned __int64	VStripVideoDemuxedSize;

CString DelayAC3(const char *szAC3File, double fDelay, CProgressCtrl *pProgressBar, HANDLE hStop, CDialog *pDlg);

using namespace std;

typedef enum STREAM_TYPE
{
	STREAM_UNKNOWN,
	STREAM_M1V,
	STREAM_M2V,
	STREAM_MPA,		// First Audio
	STREAM_AC3,
	STREAM_DTS,
	STREAM_LPCM,	// Last Audio
	STREAM_SUB
};

class CStreamInfo
{
public:
	CStreamInfo()	
	{ 
		nAudioDelay = nStreamNo = nSubStream = nPID = 0; 
		bCreateSUP = false;
		bCorrectAC3 = false;
		stStreamType = STREAM_UNKNOWN; 
		fPTSFile = fOutputFile = NULL;
		strcpy(szStreamName, "");
		bGotFirstPTS = false;
		nFirstPTS = 0;
		nLanguageID = 0;
	}

	CStreamInfo(int StreamNo, int SubStream, CString csOutputDir = "")
	{
		nAudioDelay = nPID = 0; 
		nStreamNo = StreamNo;
		bCreateSUP = false;
		bCorrectAC3 = false;
		nSubStream = SubStream;
		stStreamType = STREAM_UNKNOWN;
		fPTSFile = fOutputFile = NULL;
		strcpy(szStreamName, "");
		m_csOutputDir = csOutputDir;
		bGotFirstPTS = false;
		nFirstPTS = 0;
		nLanguageID = 0;
	}

	~CStreamInfo()
	{
		if ( fPTSFile )
		{
			fclose(fPTSFile);
			fPTSFile = NULL;
		}

		if ( fOutputFile )
		{
			if ( stStreamType == STREAM_LPCM )
			{
				CWavOutput wavout;
				wavout.CloseWavFile(fOutputFile);
			}
			fclose(fOutputFile);
			fOutputFile = NULL;
		}
	}

	void DoCreation()
	{
		if ( m_csOutputFile != "" )
		{
			if ( fPTSFile )
			{
				fclose(fPTSFile);
				fPTSFile = NULL;
			}

			if ( fOutputFile )
			{
				if ( stStreamType == STREAM_LPCM )
				{
					CWavOutput wavout;
					wavout.CloseWavFile(fOutputFile);
				}
				fclose(fOutputFile);
				fOutputFile = NULL;
			}

			if ( bCreateSUP && stStreamType == STREAM_SUB )
			{
				CString csSUPFile = m_csOutputFile;
				csSUPFile.Delete(csSUPFile.GetLength()-1);
				csSUPFile += "p"; // Change extension hack ;)

				Sub2Sup(m_csOutputFile, m_csPTSFile, csSUPFile, false);
				m_csOutputFile = csSUPFile;
			}

			// rename file to have correct Delay, may not already have it
			if ( stStreamType >= STREAM_MPA && stStreamType <= STREAM_LPCM )
			{
				if ( strrchr(m_csOutputFile, '_') )
				{
					char *szNewName = new char[m_csOutputFile.GetLength()+10];
					strcpy(szNewName, m_csOutputFile);
					*strrchr(szNewName, '_') = 0;
					sprintf(szNewName, "%s_%dms", szNewName, nAudioDelay);
					// add the ext
					if ( strrchr(m_csOutputFile, '.') )
						strcat(szNewName, strrchr(m_csOutputFile, '.'));
					rename(m_csOutputFile, szNewName);
					m_csOutputFile = szNewName;
					delete [] szNewName;
				}
			}

			if ( bCorrectAC3 && stStreamType == STREAM_AC3 )
			{
				m_csOutputFile = DelayAC3(m_csOutputFile, nAudioDelay, pProgressBar, hStop, pParentWnd);
			}

			// Now add to the total demuxed
			if ( m_csOutputFile != "" )
			{
				int fin = _open(m_csOutputFile, _O_RDONLY | _O_BINARY);
				if ( fin > 0 )
				{
					_lseeki64(fin, 0, SEEK_END);
			
					if ( nStreamNo != 0xe0 )
						VStripTotalDemuxed += (unsigned __int64)_telli64(fin);
					else
						VStripVideoDemuxedSize = (unsigned __int64)_telli64(fin);

					_close(fin);
				}
			}
		}
	}

	bool operator==(const CStreamInfo &comp) const
	{
		return ( (nStreamNo == comp.nStreamNo) && ( nSubStream == comp.nSubStream ) );
	}
	
	void SetOutputDir(CString csOutputDir)
	{
		m_csOutputDir = csOutputDir;
	}

	bool CreateOutputFile()
	{
		bool bRet = false;

		if ( !fOutputFile )
		{
			CString csOutputFile;
			CString csStream;
			CString csExt;

			switch ( stStreamType )
			{
			case STREAM_M1V:
			case STREAM_M2V:
				csStream += "MPEG_Video_";
				csExt = "m2v";
				break;
			case STREAM_AC3:
				csStream += "AC3_Audio_";
				csExt = "ac3";
				break;
			case STREAM_LPCM:
				csStream += "LPCM_Audio_";
				csExt = "wav";
				break;
			case STREAM_MPA:
				csStream += "MPA_Audio_";
				csExt = "mpa";
				break;
			case STREAM_DTS:
				csStream += "DTS_Audio_";
				csExt = "dts";
				break;
			case STREAM_SUB:
				csStream += "Sub_Stream_";
				csExt = "sub";
				break;
			case STREAM_UNKNOWN:
				csStream += "Unknown_";
				csExt = "raw";
				break;
			}

			if ( nSubStream )
				csOutputFile.Format("%s\\%s0x%X_0x%X", m_csOutputDir, csStream, nStreamNo, nSubStream);
			else
				csOutputFile.Format("%s\\%s0x%X", m_csOutputDir, csStream, nStreamNo);

			// Write out the audio delay
			if ( stStreamType != STREAM_SUB && stStreamType != STREAM_M2V )
			{
				csOutputFile += "_Delay_";
				CString csDelay;
				csDelay.Format("%d", nAudioDelay);
				csOutputFile += csDelay;
				csOutputFile += "ms";
			}

			if ( stStreamType == STREAM_SUB && bCreateSUP )
			{
				// Create PTS File
				m_csPTSFile = csOutputFile + ".pts";
				fPTSFile = fopen(m_csPTSFile, "wb");
			}

			csOutputFile += "." + csExt;
			m_csOutputFile = csOutputFile;

			fOutputFile = fopen(m_csOutputFile, "wb");
			if ( fOutputFile )
				bRet = true;
		}

		return bRet;
	}

	CString FormatText(char* fmt, ...)
	{
		char printString[200];
		va_list argp;
		va_start(argp, fmt);
		vsprintf(printString, fmt, argp);
		va_end(argp);
   
		return CString(printString);
	}

	CString GetText()
	{
		CString csStream;
			
		switch ( stStreamType )
		{
		case STREAM_M1V:
		case STREAM_M2V:
			csStream += "MPEG Video";
			break;
		case STREAM_AC3:
			csStream += "AC3 Audio";
			break;
		case STREAM_LPCM:
			csStream += "LPCM Audio";
			break;
		case STREAM_MPA:
			csStream += "MPA Audio";
			break;
		case STREAM_DTS:
			csStream += "DTS Audio";
			break;
		case STREAM_UNKNOWN:
			csStream += "Unknown ";
			break;
		}

		csStream += " Stream - ";

		if ( nStreamNo )
		{
			csStream += "StreamNo: ";
			csStream += FormatText("0x%X", nStreamNo);
			csStream += " - ";
		}

		if ( nSubStream )
		{
			csStream += "SubStreamNo: ";
			csStream += FormatText("0x%X", nSubStream);
			csStream += " - ";
		}

		if ( nPID )
		{
			csStream += "PID: ";
			csStream += FormatText("%d (0x%X)", nPID, nPID);
			csStream += " - ";
		}

		if ( nAudioDelay )
		{
			csStream += "AudioDelay: ";
			csStream += FormatText("%d", nAudioDelay);
		}

		// Remove any trailing '-'
		if ( csStream[csStream.GetLength()-2] == '-' )
			csStream.SetAt(csStream.GetLength()-2, ' ');

		return csStream;
	}

	int					nStreamNo;
	int					nSubStream;
	int					nAudioDelay;
	int					nLanguageID;
	bool				bGotFirstPTS;
	unsigned __int64	nFirstPTS;
	int					nPID;
	STREAM_TYPE			stStreamType;
	char				szLangID[3];

	// For Stream
	char			szStreamName[150];
	set<int>		PGCs;

	FILE			*fPTSFile;
	FILE			*fOutputFile;
	CString			m_csOutputDir;
	CString			m_csOutputFile;
	CString			m_csPTSFile;
	bool			bCreateSUP;
	bool			bCorrectAC3;

	HANDLE			hStop;
	CDialog			*pParentWnd;
	CProgressCtrl	*pProgressBar;
};