#include <stdio.h>
#include "wav.h"

CWavOutput::CWavOutput()
{
	m_file = NULL;
}

// Write out data
void CWavOutput::WriteWavData(void *buf, int size)
{
	fwrite(buf, 1, size, m_file);
}

// Make an unfinished RIFF structure, with null values in the lengths
int CWavOutput::CreateWavFile(char *szFN, int channels, int samplerate, int resolution)
{
    int length;

	m_file = fopen(szFN, "wb");

    /* Use a zero length for the chunk sizes for now, then modify when finished */
    WAVEAUDIOFORMAT format;

    format.format_tag = 1;
    format.channels = channels;
    format.samplerate = samplerate;
    format.bits_per_sample = resolution;
    format.blockalign = channels * (resolution/8);
    format.bytes_per_second = format.samplerate * format.blockalign;

    fseek(m_file, 0, SEEK_SET);

    fwrite("RIFF\0\0\0\0WAVEfmt ", sizeof(char), 16, m_file); /* Write RIFF, WAVE, and fmt  headers */

    length = 16;
    fwrite(&length, 1, sizeof(long), m_file); /* Length of Format (always 16) */
    fwrite(&format, 1, sizeof(format), m_file);

    fwrite("data\0\0\0\0", sizeof(char), 8, m_file); /* Write data chunk */

    return 0;
}

int CWavOutput::CreateWavFile(FILE *file, int channels, int samplerate, int resolution)
{
    int length;

	/* Use a zero length for the chunk sizes for now, then modify when finished */
    WAVEAUDIOFORMAT format;

    format.format_tag = 1;
    format.channels = channels;
    format.samplerate = samplerate;
    format.bits_per_sample = resolution;
    format.blockalign = channels * (resolution/8);
    format.bytes_per_second = format.samplerate * format.blockalign;

    fseek(file, 0, SEEK_SET);

    fwrite("RIFF\0\0\0\0WAVEfmt ", sizeof(char), 16, file); /* Write RIFF, WAVE, and fmt  headers */

    length = 16;
    fwrite(&length, 1, sizeof(long), file); /* Length of Format (always 16) */
    fwrite(&format, 1, sizeof(format), file);

    fwrite("data\0\0\0\0", sizeof(char), 8, file); /* Write data chunk */

    return 0;
}

/* Update the RIFF structure with proper values. CreateWavHeader must be called first */
int CWavOutput::CloseWavFile(FILE *file)
{
    /*fpos_t*/int filelen, riff_length, data_length;

    /* Get the length of the file */

    if(!file)
        return -1;

    fseek( file, 0, SEEK_END );
    filelen = ftell( file );

    if(filelen <= 0)
        return -1;

    riff_length = filelen - 8;
    data_length = filelen - 44;

    fseek(file, 4, SEEK_SET);
    fwrite(&riff_length, 1, sizeof(long), file);

    fseek(file, 40, SEEK_SET);
    fwrite(&data_length, 1, sizeof(long), file);

    /* reset file position for appending data */
    fseek(file, 0, SEEK_END);

    return 0;
}


int CWavOutput::CloseWavFile()
{
    /*fpos_t*/int filelen, riff_length, data_length;

    /* Get the length of the file */

    if(!m_file)
        return -1;

    fseek( m_file, 0, SEEK_END );
    filelen = ftell( m_file );

    if(filelen <= 0)
        return -1;

    riff_length = filelen - 8;
    data_length = filelen - 44;

    fseek(m_file, 4, SEEK_SET);
    fwrite(&riff_length, 1, sizeof(long), m_file);

    fseek(m_file, 40, SEEK_SET);
    fwrite(&data_length, 1, sizeof(long), m_file);

    /* reset file position for appending data */
    fseek(m_file, 0, SEEK_END);

    return 0;
}


